@@ -1,35 +1,62 @@
 /* bignumc.c - C routines equivalent to ASM routines in bignuma.asm */
-/* Wesley Loewer's Big Numbers.        (C) 1994, Wesley B. Loewer   */
+
+/*
+Wesley Loewer's Big Numbers.        (C) 1994-95, Wesley B. Loewer
+*/
 
 #include <memory.h>
-#include "prototyp.h"
-
-#ifndef _BIGNUM_H
-#include "bignum.h"
-#endif
-
-#ifdef sign
-#undef sign
-#endif
-
+  /* see Fractint.c for a description of the "include"  hierarchy */
+#include "port.h"
+#include "big.h"
 
 /********************************************************************
  The following code contains the C versions of the routines from the
  file BIGNUMA.ASM.  It is provided here for portibility and for clarity.
 *********************************************************************/
 
+/********************************************************************
+ Note:  The C code must be able to detect over/underflow.  Therefore
+ 32 bit integers must be used when doing 16 bit math.  All we really
+ need is one more bit, such as is provided in asm with the carry bit.
+ Functions that don't need the test for over/underflow, such as cmp_bn()
+ and is_bn_not_zero(), can use 32 bit integers as as long as bnstep
+ is set to 4.
+
+ The 16/32 bit compination of integer sizes could be increased to
+ 32/64 bit to improve efficiency, but since many compilers don't offer
+ 64 bit integers, this option was not included.
+
+*********************************************************************/
+
 /********************************************************************/
 /* r = 0 */
 bn_t clear_bn(bn_t r)
     {
-    return _fmemset( r, 0, bnlength); /* set array to zero */
+#ifdef BIG_BASED
+    _fmemset( r, 0, bnlength); /* set array to zero */
+#else
+#ifdef BIG_FAR
+    _fmemset( r, 0, bnlength); /* set array to zero */
+#else
+    memset( r, 0, bnlength); /* set array to zero */
+#endif
+#endif
+    return r;
     }
 
 /********************************************************************/
 /* r = max positive value */
 bn_t max_bn(bn_t r)
     {
+#ifdef BIG_BASED
     _fmemset( r, 0xFF, bnlength-1); /* set to max values */
+#else
+#ifdef BIG_FAR
+    _fmemset( r, 0xFF, bnlength-1); /* set to max values */
+#else
+    memset( r, 0xFF, bnlength-1); /* set to max values */
+#endif
+#endif
     r[bnlength-1] = 0x7F;  /* turn off the sign bit */
     return r;
     }
@@ -38,35 +65,65 @@ bn_t max_bn(bn_t r)
 /* r = n */
 bn_t copy_bn(bn_t r, bn_t n)
     {
-    return _fmemcpy( r, n, bnlength);
+#ifdef BIG_BASED
+    _fmemcpy( r, n, bnlength);
+#else
+#ifdef BIG_FAR
+    _fmemcpy( r, n, bnlength);
+#else
+    memcpy( r, n, bnlength);
+#endif
+#endif
+    return r;
     }
 
 /***************************************************************************/
 /* n1 != n2 ?                                                              */
 /* RETURNS:                                                                */
 /*  if n1 == n2 returns 0                                                  */
-/*  if n1 > n2 returns a positive (steps left to go when mismatch occured) */
-/*  if n1 < n2 returns a negative (steps left to go when mismatch occured) */
+/*  if n1 > n2 returns a positive (bytes left to go when mismatch occured) */
+/*  if n1 < n2 returns a negative (bytes left to go when mismatch occured) */
 int cmp_bn(bn_t n1, bn_t n2)
     {
     int i;
+    S16 Svalue1, Svalue2;
+    U16 value1, value2;
 
     /* two bytes at a time */
     /* signed comparison for msb */
-    if (accessS16((S16 far *)(n1+bnlength-2)) >
-	    accessS16((S16 far *)(n2+bnlength-2)))
-        return bnlength;
-    else if (accessS16((S16 far *)(n1+bnlength-2)) <
-	    accessS16((S16 far *)(n2+bnlength-2)))
-        return -(bnlength);
+    if ( (Svalue1=big_accessS16((S16 BIGDIST *)(n1+bnlength-2))) >
+         (Svalue2=big_accessS16((S16 BIGDIST *)(n2+bnlength-2))) )
+        { /* now determine which of the two bytes was different */
+        if ( (S16)(Svalue1&0xFF00) > (S16)(Svalue2&0xFF00) ) /* compare just high bytes */
+            return (bnlength); /* high byte was different */
+        else
+            return (bnlength-1); /* low byte was different */
+        }
+    else if (Svalue1 < Svalue2)
+        { /* now determine which of the two bytes was different */
+        if ( (S16)(Svalue1&0xFF00) < (S16)(Svalue2&0xFF00) ) /* compare just high bytes */
+            return -(bnlength); /* high byte was different */
+        else
+            return -(bnlength-1); /* low byte was different */
+        }
 
     /* unsigned comparison for the rest */
     for (i=bnlength-4; i>=0; i-=2)
         {
-        if (access16(n1+i) > access16(n2+i))
-            return i+2;
-        else if (access16(n1+i) < access16(n2+i))
-            return -(i+2);
+        if ( (value1=big_access16(n1+i)) > (value2=big_access16(n2+i)) )
+            { /* now determine which of the two bytes was different */
+            if ( (value1&0xFF00) > (value2&0xFF00) ) /* compare just high bytes */
+                return (i+2); /* high byte was different */
+            else
+                return (i+1); /* low byte was different */
+            }
+        else if (value1 < value2)
+            { /* now determine which of the two bytes was different */
+            if ( (value1&0xFF00) < (value2&0xFF00) ) /* compare just high bytes */
+                return -(i+2); /* high byte was different */
+            else
+                return -(i+1); /* low byte was different */
+            }
         }
     return 0;
     }
@@ -89,7 +146,7 @@ int is_bn_not_zero(bn_t n)
 
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
-        if (access16(n+i) != 0)
+        if (big_access16(n+i) != 0)
             return 1;
     return 0;
     }
@@ -104,8 +161,8 @@ bn_t add_bn(bn_t r, bn_t n1, bn_t n2)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        sum += (U32)access16(n1+i) + (U32)access16(n2+i); /* add 'em up */
-        set16(r+i, (U16)sum);   /* store the lower 2 bytes */
+        sum += (U32)big_access16(n1+i) + (U32)big_access16(n2+i); /* add 'em up */
+        big_set16(r+i, (U16)sum);   /* store the lower 2 bytes */
         sum >>= 16; /* shift the overflow for next time */
         }
     return r;
@@ -121,8 +178,8 @@ bn_t add_a_bn(bn_t r, bn_t n)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        sum += (U32)access16(r+i) + (U32)access16(n+i); /* add 'em up */
-        set16(r+i, (U16)sum);   /* store the lower 2 bytes */
+        sum += (U32)big_access16(r+i) + (U32)big_access16(n+i); /* add 'em up */
+        big_set16(r+i, (U16)sum);   /* store the lower 2 bytes */
         sum >>= 16; /* shift the overflow for next time */
         }
     return r;
@@ -138,8 +195,8 @@ bn_t sub_bn(bn_t r, bn_t n1, bn_t n2)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        diff = (U32)access16(n1+i) - ((U32)access16(n2+i)-(S32)(S16)diff); /* subtract with borrow */
-        set16(r+i, (U16)diff);   /* store the lower 2 bytes */
+        diff = (U32)big_access16(n1+i) - ((U32)big_access16(n2+i)-(S32)(S16)diff); /* subtract with borrow */
+        big_set16(r+i, (U16)diff);   /* store the lower 2 bytes */
         diff >>= 16; /* shift the underflow for next time */
         }
     return r;
@@ -155,8 +212,8 @@ bn_t sub_a_bn(bn_t r, bn_t n)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        diff = (U32)access16(r+i) - ((U32)access16(n+i)-(S32)(S16)diff); /* subtract with borrow */
-        set16(r+i, (U16)diff);   /* store the lower 2 bytes */
+        diff = (U32)big_access16(r+i) - ((U32)big_access16(n+i)-(S32)(S16)diff); /* subtract with borrow */
+        big_set16(r+i, (U16)diff);   /* store the lower 2 bytes */
         diff >>= 16; /* shift the underflow for next time */
         }
     return r;
@@ -173,15 +230,15 @@ bn_t neg_bn(bn_t r, bn_t n)
     /* two bytes at a time */
     for (i=0; neg != 0 && i<bnlength; i+=2)
         {
-        t_short = ~access16(n+i);
+        t_short = ~big_access16(n+i);
         neg += ((U32)t_short); /* two's complement */
-        set16(r+i, (U16)neg);   /* store the lower 2 bytes */
+        big_set16(r+i, (U16)neg);   /* store the lower 2 bytes */
         neg >>= 16; /* shift the sign bit for next time */
         }
     /* if neg was 0, then just "not" the rest */
     for (; i<bnlength; i+=2)
-        { /* notice that access16() and set16() are not needed here */
-        *(U16 far *)(r+i) = ~*(U16 far *)(n+i); /* toggle all the bits */
+        { /* notice that big_access16() and big_set16() are not needed here */
+        *(U16 BIGDIST *)(r+i) = ~*(U16 BIGDIST *)(n+i); /* toggle all the bits */
         }
     return r;
     }
@@ -197,15 +254,15 @@ bn_t neg_a_bn(bn_t r)
     /* two bytes at a time */
     for (i=0; neg != 0 && i<bnlength; i+=2)
         {
-	t_short = ~access16(r+i);
+        t_short = ~big_access16(r+i);
         neg += ((U32)t_short); /* two's complement */
-        set16(r+i, (U16)neg);   /* store the lower 2 bytes */
+        big_set16(r+i, (U16)neg);   /* store the lower 2 bytes */
         neg >>= 16; /* shift the sign bit for next time */
         }
     /* if neg was 0, then just "not" the rest */
     for (; i<bnlength; i+=2)
-        { /* notice that access16() and set16() are not needed here */
-        *(U16 far *)(r+i) = ~*(U16 far *)(r+i); /* toggle all the bits */
+        { /* notice that big_access16() and big_set16() are not needed here */
+        *(U16 BIGDIST *)(r+i) = ~*(U16 BIGDIST *)(r+i); /* toggle all the bits */
         }
     return r;
     }
@@ -220,8 +277,8 @@ bn_t double_bn(bn_t r, bn_t n)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        prod += (U32)access16(n+i)<<1 ; /* double it */
-        set16(r+i, (U16)prod);   /* store the lower 2 bytes */
+        prod += (U32)big_access16(n+i)<<1 ; /* double it */
+        big_set16(r+i, (U16)prod);   /* store the lower 2 bytes */
         prod >>= 16; /* shift the overflow for next time */
         }
     return r;
@@ -237,8 +294,8 @@ bn_t double_a_bn(bn_t r)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        prod += (U32)access16(r+i)<<1 ; /* double it */
-        set16(r+i, (U16)prod);   /* store the lower 2 bytes */
+        prod += (U32)big_access16(r+i)<<1 ; /* double it */
+        big_set16(r+i, (U16)prod);   /* store the lower 2 bytes */
         prod >>= 16; /* shift the overflow for next time */
         }
     return r;
@@ -255,15 +312,15 @@ bn_t half_bn(bn_t r, bn_t n)
 
     /* start with an arithmetic shift */
     i=bnlength-2;
-    quot += (U32)(((S32)(S16)access16(n+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
-    set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
+    quot += (U32)(((S32)(S16)big_access16(n+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
+    big_set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
     quot <<= 16; /* shift the underflow for next time */
 
     for (i=bnlength-4; i>=0; i-=2)
         {
         /* looks wierd, but properly sign extends argument */
-        quot += (U32)(((U32)access16(n+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
-        set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
+        quot += (U32)(((U32)big_access16(n+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
+        big_set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
         quot <<= 16; /* shift the underflow for next time */
         }
 
@@ -281,15 +338,15 @@ bn_t half_a_bn(bn_t r)
 
     /* start with an arithmetic shift */
     i=bnlength-2;
-    quot += (U32)(((S32)(S16)access16(r+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
-    set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
+    quot += (U32)(((S32)(S16)big_access16(r+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
+    big_set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
     quot <<= 16; /* shift the underflow for next time */
 
     for (i=bnlength-4; i>=0; i-=2)
         {
         /* looks wierd, but properly sign extends argument */
-        quot += (U32)(((U32)(U16)access16(r+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
-        set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
+        quot += (U32)(((U32)(U16)big_access16(r+i)<<16)>>1) ; /* shift to upper 2 bytes and half it */
+        big_set16(r+i, (U16)(quot>>16));   /* store the upper 2 bytes */
         quot <<= 16; /* shift the underflow for next time */
         }
     return r;
@@ -310,7 +367,8 @@ bn_t unsafe_full_mult_bn(bn_t r, bn_t n1, bn_t n2)
 
     if ((sign1 = is_bn_neg(n1)) != 0) /* =, not == */
         neg_a_bn(n1);
-    if (!(samevar = (n1 == n2))) /* check to see if they're the same pointer */
+        samevar = (n1 == n2);
+        if (!samevar) /* check to see if they're the same pointer */
         if ((sign2 = is_bn_neg(n2)) != 0) /* =, not == */
             neg_a_bn(n2);
 
@@ -326,19 +384,19 @@ bn_t unsafe_full_mult_bn(bn_t r, bn_t n1, bn_t n2)
         n2p = n2;
         for (j = 0; j < steps; j++)
             {
-            prod = (U32)access16(n1p) * (U32)access16(n2p); /* U16*U16=U32 */
-            sum = (U32)access16(rp2) + prod; /* add to previous, including overflow */
-            set16(rp2, (U16)sum); /* save the lower 2 bytes */
+            prod = (U32)big_access16(n1p) * (U32)big_access16(n2p); /* U16*U16=U32 */
+            sum = (U32)big_access16(rp2) + prod; /* add to previous, including overflow */
+            big_set16(rp2, (U16)sum); /* save the lower 2 bytes */
             sum >>= 16;             /* keep just the upper 2 bytes */
             rp3 = rp2 + 2;          /* move over 2 bytes */
-            sum += access16(rp3);     /* add what was the upper two bytes */
-            set16(rp3 ,(U16)sum); /* save what was the upper two bytes */
+            sum += big_access16(rp3);     /* add what was the upper two bytes */
+            big_set16(rp3 ,(U16)sum); /* save what was the upper two bytes */
             sum >>= 16;             /* keep just the overflow */
             for (k=0; sum != 0 && k<carry_steps; k++)
                 {
                 rp3 += 2;               /* move over 2 bytes */
-                sum += access16(rp3);     /* add to what was the overflow */
-                set16(rp3, (U16)sum); /* save what was the overflow */
+                sum += big_access16(rp3);     /* add to what was the overflow */
+                big_set16(rp3, (U16)sum); /* save what was the overflow */
                 sum >>= 16;             /* keep just the new overflow */
                 }
             n2p += 2;       /* to next word */
@@ -378,7 +436,8 @@ bn_t unsafe_mult_bn(bn_t r, bn_t n1, bn_t n2)
     bnl = bnlength;
     if ((sign1 = is_bn_neg(n1)) != 0) /* =, not == */
         neg_a_bn(n1);
-    if (!(samevar = (n1 == n2))) /* check to see if they're the same pointer */
+        samevar = (n1 == n2);
+        if (!samevar) /* check to see if they're the same pointer */
         if ((sign2 = is_bn_neg(n2)) != 0) /* =, not == */
             neg_a_bn(n2);
     n1p = n1;
@@ -397,19 +456,19 @@ bn_t unsafe_mult_bn(bn_t r, bn_t n1, bn_t n2)
         n2p = n2;
         for (j=0; j<steps; j++)
             {
-            prod = (U32)access16(n1p) * (U32)access16(n2p); /* U16*U16=U32 */
-            sum = (U32)access16(rp2) + prod; /* add to previous, including overflow */
-            set16(rp2, (U16)sum); /* save the lower 2 bytes */
+            prod = (U32)big_access16(n1p) * (U32)big_access16(n2p); /* U16*U16=U32 */
+            sum = (U32)big_access16(rp2) + prod; /* add to previous, including overflow */
+            big_set16(rp2, (U16)sum); /* save the lower 2 bytes */
             sum >>= 16;             /* keep just the upper 2 bytes */
             rp3 = rp2 + 2;          /* move over 2 bytes */
-            sum += access16(rp3);     /* add what was the upper two bytes */
-            set16(rp3, (U16)sum); /* save what was the upper two bytes */
+            sum += big_access16(rp3);     /* add what was the upper two bytes */
+            big_set16(rp3, (U16)sum); /* save what was the upper two bytes */
             sum >>= 16;             /* keep just the overflow */
             for (k=0; sum != 0 && k<carry_steps; k++)
                 {
                 rp3 += 2;               /* move over 2 bytes */
-                sum += access16(rp3);     /* add to what was the overflow */
-                set16(rp3, (U16)sum); /* save what was the overflow */
+                sum += big_access16(rp3);     /* add to what was the overflow */
+                big_set16(rp3, (U16)sum); /* save what was the overflow */
                 sum >>= 16;             /* keep just the new overflow */
                 }
             n2p += 2;                   /* increase by two bytes */
@@ -478,19 +537,19 @@ bn_t unsafe_full_square_bn(bn_t r, bn_t n)
             n2p = n1p + 2;  /* set n2p pointer to 1 step beyond n1p */
             for (j=0; j<steps; j++)
                 {
-                prod = (U32)access16(n1p) * (U32)access16(n2p); /* U16*U16=U32 */
-                sum = (U32)access16(rp2) + prod; /* add to previous, including overflow */
-                set16(rp2, (U16)sum); /* save the lower 2 bytes */
+                prod = (U32)big_access16(n1p) * (U32)big_access16(n2p); /* U16*U16=U32 */
+                sum = (U32)big_access16(rp2) + prod; /* add to previous, including overflow */
+                big_set16(rp2, (U16)sum); /* save the lower 2 bytes */
                 sum >>= 16;             /* keep just the upper 2 bytes */
                 rp3 = rp2 + 2;          /* move over 2 bytes */
-                sum += access16(rp3);     /* add what was the upper two bytes */
-                set16(rp3, (U16)sum); /* save what was the upper two bytes */
+                sum += big_access16(rp3);     /* add what was the upper two bytes */
+                big_set16(rp3, (U16)sum); /* save what was the upper two bytes */
                 sum >>= 16;             /* keep just the overflow */
                 for (k=0; sum != 0 && k<carry_steps; k++)
                     {
                     rp3 += 2;               /* move over 2 bytes */
-                    sum += access16(rp3);     /* add to what was the overflow */
-                    set16(rp3, (U16)sum); /* save what was the overflow */
+                    sum += big_access16(rp3);     /* add to what was the overflow */
+                    big_set16(rp3, (U16)sum); /* save what was the overflow */
                     sum >>= 16;             /* keep just the new overflow */
                     }
                 n2p += 2;       /* increase by two bytes */
@@ -517,19 +576,19 @@ bn_t unsafe_full_square_bn(bn_t r, bn_t n)
     for (i=0; i<steps; i++)
         {
         /* square it */
-        prod = (U32)access16(n1p) * (U32)access16(n1p); /* U16*U16=U32 */
-        sum = (U32)access16(rp1) + prod; /* add to previous, including overflow */
-        set16(rp1, (U16)sum); /* save the lower 2 bytes */
+        prod = (U32)big_access16(n1p) * (U32)big_access16(n1p); /* U16*U16=U32 */
+        sum = (U32)big_access16(rp1) + prod; /* add to previous, including overflow */
+        big_set16(rp1, (U16)sum); /* save the lower 2 bytes */
         sum >>= 16;             /* keep just the upper 2 bytes */
         rp3 = rp1 + 2;          /* move over 2 bytes */
-        sum += access16(rp3);     /* add what was the upper two bytes */
-        set16(rp3, (U16)sum); /* save what was the upper two bytes */
+        sum += big_access16(rp3);     /* add what was the upper two bytes */
+        big_set16(rp3, (U16)sum); /* save what was the upper two bytes */
         sum >>= 16;             /* keep just the overflow */
         for (k=0; sum != 0 && k<carry_steps; k++)
             {
             rp3 += 2;               /* move over 2 bytes */
-            sum += access16(rp3);     /* add to what was the overflow */
-            set16(rp3, (U16)sum); /* save what was the overflow */
+            sum += big_access16(rp3);     /* add to what was the overflow */
+            big_set16(rp3, (U16)sum); /* save what was the overflow */
             sum >>= 16;             /* keep just the new overflow */
             }
         n1p += 2;       /* increase by 2 bytes */
@@ -564,7 +623,7 @@ bn_t unsafe_square_bn(bn_t r, bn_t n)
 /* This whole procedure would be a great deal simpler if we could assume that */
 /* rlength < 2*bnlength (that is, not =).  Therefore, we will take the        */
 /* easy way out and call full_square_bn() if it is.                           */
-    if (rlength == bnlength<<1) /* rlength == 2*bnlength */
+    if (rlength == (bnlength<<1)) /* rlength == 2*bnlength */
         return unsafe_full_square_bn(r, n);    /* call full_square_bn() and quit */
 
     if (is_bn_neg(n))  /* don't need to keep track of sign since the */
@@ -591,19 +650,19 @@ bn_t unsafe_square_bn(bn_t r, bn_t n)
             {
             for (j=0; j<steps; j++)
                 {
-                prod = (U32)access16(n1p) * (U32)access16(n2p); /* U16*U16=U32 */
-                sum = (U32)access16(rp2) + prod; /* add to previous, including overflow */
-                set16(rp2, (U16)sum); /* save the lower 2 bytes */
+                prod = (U32)big_access16(n1p) * (U32)big_access16(n2p); /* U16*U16=U32 */
+                sum = (U32)big_access16(rp2) + prod; /* add to previous, including overflow */
+                big_set16(rp2, (U16)sum); /* save the lower 2 bytes */
                 sum >>= 16;             /* keep just the upper 2 bytes */
                 rp3 = rp2 + 2;          /* move over 2 bytes */
-                sum += access16(rp3);     /* add what was the upper two bytes */
-                set16(rp3, (U16)sum); /* save what was the upper two bytes */
+                sum += big_access16(rp3);     /* add what was the upper two bytes */
+                big_set16(rp3, (U16)sum); /* save what was the upper two bytes */
                 sum >>= 16;             /* keep just the overflow */
                 for (k=0; sum != 0 && k<carry_steps; k++)
                     {
                     rp3 += 2;               /* move over 2 bytes */
-                    sum += access16(rp3);     /* add to what was the overflow */
-                    set16(rp3, (U16)sum); /* save what was the overflow */
+                    sum += big_access16(rp3);     /* add to what was the overflow */
+                    big_set16(rp3, (U16)sum); /* save what was the overflow */
                     sum >>= 16;             /* keep just the new overflow */
                     }
                 n2p += 2;       /* increase by 2-byte word size */
@@ -656,19 +715,19 @@ bn_t unsafe_square_bn(bn_t r, bn_t n)
     for (; i>0; i--)
         {
         /* square it */
-        prod = (U32)access16(n1p) * (U32)access16(n1p); /* U16*U16=U32 */
-        sum = (U32)access16(rp1) + prod; /* add to previous, including overflow */
-        set16(rp1, (U16)sum); /* save the lower 2 bytes */
+        prod = (U32)big_access16(n1p) * (U32)big_access16(n1p); /* U16*U16=U32 */
+        sum = (U32)big_access16(rp1) + prod; /* add to previous, including overflow */
+        big_set16(rp1, (U16)sum); /* save the lower 2 bytes */
         sum >>= 16;             /* keep just the upper 2 bytes */
         rp3 = rp1 + 2;          /* move over 2 bytes */
-        sum += access16(rp3);     /* add what was the upper two bytes */
-        set16(rp3, (U16)sum); /* save what was the upper two bytes */
+        sum += big_access16(rp3);     /* add what was the upper two bytes */
+        big_set16(rp3, (U16)sum); /* save what was the upper two bytes */
         sum >>= 16;             /* keep just the overflow */
         for (k=0; sum != 0 && k<carry_steps; k++)
             {
             rp3 += 2;               /* move over 2 bytes */
-            sum += access16(rp3);     /* add to what was the overflow */
-            set16(rp3, (U16)sum); /* save what was the overflow */
+            sum += big_access16(rp3);     /* add to what was the overflow */
+            big_set16(rp3, (U16)sum); /* save what was the overflow */
             sum >>= 16;             /* keep just the new overflow */
             }
         n1p += 2;
@@ -688,8 +747,8 @@ bn_t mult_bn_int(bn_t r, bn_t n, U16 u)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        prod += (U32)access16(n+i) * u ; /* n*u */
-        set16(r+i, (U16)prod);   /* store the lower 2 bytes */
+        prod += (U32)big_access16(n+i) * u ; /* n*u */
+        big_set16(r+i, (U16)prod);   /* store the lower 2 bytes */
         prod >>= 16; /* shift the overflow for next time */
         }
     return r;
@@ -705,8 +764,8 @@ bn_t mult_a_bn_int(bn_t r, U16 u)
     /* two bytes at a time */
     for (i=0; i<bnlength; i+=2)
         {
-        prod += (U32)access16(r+i) * u ; /* r*u */
-        set16(r+i, (U16)prod);   /* store the lower 2 bytes */
+        prod += (U32)big_access16(r+i) * u ; /* r*u */
+        big_set16(r+i, (U16)prod);   /* store the lower 2 bytes */
         prod >>= 16; /* shift the overflow for next time */
         }
     return r;
@@ -735,10 +794,10 @@ bn_t unsafe_div_bn_int(bn_t r, bn_t n,  U16 u)
     /* two bytes at a time */
     for (i=bnlength-2; i>=0; i-=2)
         {
-        full_number = ((U32)rem<<16) + (U32)access16(n+i);
+        full_number = ((U32)rem<<16) + (U32)big_access16(n+i);
         quot = (U16)(full_number / u);
         rem  = (U16)(full_number % u);
-        set16(r+i, quot);
+        big_set16(r+i, quot);
         }
 
     if (sign)
@@ -769,10 +828,10 @@ bn_t div_a_bn_int(bn_t r, U16 u)
     /* two bytes at a time */
     for (i=bnlength-2; i>=0; i-=2)
         {
-        full_number = ((U32)rem<<16) + (U32)access16(r+i);
+        full_number = ((U32)rem<<16) + (U32)big_access16(r+i);
         quot = (U16)(full_number / u);
         rem  = (U16)(full_number % u);
-        set16(r+i, quot);
+        big_set16(r+i, quot);
         }
 
     if (sign)
@@ -822,3 +881,108 @@ LDBL bntofloat(bn_t n)
     return f;
     }
 
+/*****************************************/
+/* the following used to be in bigfltc.c */
+
+/********************************************************************/
+/* r = 0 */
+bf_t clear_bf(bf_t r)
+    {
+    _fmemset( r, 0, bflength+2); /* set array to zero */
+    return r;
+    }
+
+/********************************************************************/
+/* r = n */
+bf_t copy_bf(bf_t r, bf_t n)
+    {
+    _fmemcpy( r, n, bflength+2);
+    return r;
+    }
+
+/*********************************************************************/
+/*  b = f                                                            */
+/*  Converts a double to a bigfloat                                  */
+bf_t floattobf(bf_t r, LDBL f)
+    {
+    int power;
+    int bnl, il;
+    if (f == 0)
+        {
+        clear_bf(r);
+        return r;
+        }
+
+    /* remove the exp part */
+    f = extract_256(f, &power);
+
+    bnl = bnlength;
+    bnlength = bflength;
+    il = intlength;
+    intlength = 2;
+    floattobn(r, f);
+    bnlength = bnl;
+    intlength = il;
+
+    big_set16(r + bflength, (S16)power); /* exp */
+
+    return r;
+    }
+
+/*********************************************************************/
+/*  b = f                                                            */
+/*  Converts a double to a bigfloat                                  */
+bf_t floattobf1(bf_t r, LDBL f)
+    {
+    char msg[80];
+#ifdef USE_LONG_DOUBLE
+    sprintf(msg,"%-.22Le",f);
+#else
+    sprintf(msg,"%-.22le",f);
+#endif
+    strtobf(r,msg);
+    return r;
+    }
+
+/*********************************************************************/
+/*  f = b                                                            */
+/*  Converts a bigfloat to a double                                 */
+LDBL bftofloat(bf_t n)
+    {
+    int power;
+    int bnl, il;
+    LDBL f;
+
+    bnl = bnlength;
+    bnlength = bflength;
+    il = intlength;
+    intlength = 2;
+    f = bntofloat(n);
+    bnlength = bnl;
+    intlength = il;
+
+    power = (S16)big_access16(n + bflength);
+    f = scale_256(f,power);
+
+    return f;
+    }
+
+/********************************************************************/
+/* extracts the mantissa and exponent of f                          */
+/* finds m and n such that 1<=|m|<256 and f = m*256^n               */
+/* n is stored in *exp_ptr and m is returned, sort of like frexp()  */
+LDBL extract_256(LDBL f, int *exp_ptr)
+   {
+   return extract_value(f, 256, exp_ptr);
+   }
+
+/********************************************************************/
+/* calculates and returns the value of f*256^n                      */
+/* sort of like ldexp()                                             */
+/*                                                                  */
+/* n must be in the range -2^12 <= n < 2^12 (2^12=4096),            */
+/* which should not be a problem                                    */
+LDBL scale_256( LDBL f, int n )
+   {
+   return scale_value( f, 256, n );
+   }
