@@ -12,16 +12,8 @@
 #include "prototyp.h"
 #include "fractype.h"
 
-#ifdef WINFRACT
-extern int OperCancelled;
-void OpenStatusBox(void);
-void UpdateStatusBox(unsigned long Partial, unsigned long Total);
-void CloseStatusBox(void);
-#endif
-
-static int inittable(void);
-static int _fastcall shftwrite(BYTE * color, int numcolors);
-static int _fastcall raster(unsigned int);
+static int compress(int rowlimit);
+static int _fastcall shftwrite(BYTE far * color, int numcolors);
 static int _fastcall extend_blk_len(int datalen);
 static int _fastcall put_extend_blk(int block_id, int block_len, char far * block_data);
 static int _fastcall store_item_name(char *);
@@ -34,78 +26,44 @@ int decode_fractal_info();
 /*
                         Save-To-Disk Routines (GIF)
 
-GIF and 'Graphics Interchange Format' are trademarks (tm) of Compuserve
-Incorporated, an H&R Block Company.
-
-
 The following routines perform the GIF encoding when the 's' key is pressed.
-The routines refer to several variables that are declared elsewhere
-[colors, xdots, ydots, and 'dacbox'], and rely on external routines to
-actually read and write screen pixels [getcolor(x,y) and putcolor(x,y,color)].
-(Writing pixels is just stuffed in here as a sort of visual status report,
-and has nothing to do with any GIF function.)   They also rely on the
-existence of an externally-defined 64K dataspace and they use the routines
-'toextra()' and 'cmpextra()' to deal with that dataspace (in the same manner
-as 'memcpy()' and 'memcmp()' would).   Otherwise, they perform a generic
-GIF-encoder function.
 
-Note that these routines use small string- and hash-tables, and "flush"
-the GIF entries whenever the hash-table gets two-thirds full or the string
-table gets full.   They also use the GIF encoding technique of limiting the
-encoded string length to a specific size, "adding" a string to the hash table
-at that point even if a matching string exists ("adding" is in quotes, because
-if a matching string exists we can increment the code counter but safely throw
-the duplicate string away, saving both string space and a hash table entry).
+The compression logic in this file has been replaced by the classic
+UNIX compress code. We have extensively modified the sources to fit
+Fractint's needs, but have left the original credits where they
+appear. Thanks to the original authors for making available these
+classic and reliable sources. Of course, they our not responsible for 
+all the changes we have made to integrate their sources into Fractint.
 
-This results in relatively good speed and small data space, but at the
-expense of compression efficiency (filesize).   These trade-offs could be
-adjusted by modifying the #DEFINEd variables below.
+MEMORY ALLOCATION
 
-Note that the 'strlocn' and 'teststring' routines are declared
-to be external just so that they can be defined (and the space re-used)
-elsewhere.  The actual declarations are in the assembler code.
+There are two large arrays:
+
+   long htab[HSIZE]              (5003*4 = 20012 bytes)
+   unsigned short codetab[HSIZE] (5003*2 = 10006 bytes)
+   
+At the moment these arrays reuse extraseg and strlocn, respectively.
 
 */
 
-#define MAXTEST   100           /* maximum single string length */
-#define MAXSTRING 60000L        /* total space reserved for strings */
-                                /* maximum number of strings available */
-#define MAXENTRY  5003          /* (a prime number is best for hashing) */
-
-#ifdef XFRACT
-unsigned int strlocn[10240];
-BYTE teststring[MAXTEST];
-BYTE block[266];                /* GIF-encoded blocks go here */
-#endif
-
 static int numsaves = 0;        /* For adjusting 'save-to-disk' filenames */
-
-static FILE *out;
+static FILE *g_outfile;
 static int last_colorbar;
 static int save16bit;
 static int outcolor1s, outcolor2s;
+static int startbits;
 
-long max_nextentry;
-
-static int lentest, lastentry, numentries, numrealentries;
-static unsigned int nextentry;
-static int clearcode, endcode;
-static unsigned int hashcode;
-
-static BYTE blockcount;
-static int startbits, codebits, bytecount, bitcount;
-
-static BYTE paletteBW[] =
+static BFCODE paletteBW[] =
 {                               /* B&W palette */
    0, 0, 0, 63, 63, 63,
 };
 
-static BYTE paletteCGA[] =
+static BFCODE paletteCGA[] =
 {                               /* 4-color (CGA) palette  */
    0, 0, 0, 21, 63, 63, 63, 21, 63, 63, 63, 63,
 };
 
-static BYTE paletteEGA[] =
+static BFCODE paletteEGA[] =
 {                               /* 16-color (EGA/CGA) pal */
    0, 0, 0, 0, 0, 42, 0, 42, 0, 0, 42, 42,
    42, 0, 0, 42, 0, 42, 42, 21, 0, 42, 42, 42,
@@ -120,8 +78,8 @@ int savetodisk(char *filename)      /* save-to-disk routine */
    char tmpfile[FILE_MAX_PATH];
    char *period;
    int newfile;
-   int i, j, outcolor1, outcolor2, interrupted;
- restart:
+   int i, j, interrupted, outcolor1, outcolor2;
+restart:
 
    save16bit = disk16bit;
    if (gif87a_flag)             /* not storing non-standard fractal info */
@@ -176,7 +134,7 @@ int savetodisk(char *filename)      /* save-to-disk routine */
    if (resave_flag == 2)        /* final save of savetime set? */
       resave_flag = 0;
 
-   if ((out = fopen(tmpfile, "wb")) == NULL)
+   if ((g_outfile = fopen(tmpfile, "wb")) == NULL)
    {
       sprintf(tmpmsg, s_cantcreate, tmpfile);
       stopmsg(0, tmpmsg);
@@ -209,7 +167,7 @@ int savetodisk(char *filename)      /* save-to-disk routine */
 
    busy = 0;
 
-   fclose(out);
+   fclose(g_outfile);
 
    if (interrupted)
    {
@@ -254,9 +212,10 @@ int savetodisk(char *filename)      /* save-to-disk routine */
                   outcolor2 = 0;
             }
             for (i = 0; 250 * i < xdots; i++)
-            {                   /* clear vert status bars */
+            {  /* clear vert status bars */
                putcolor(i, j, getcolor(i, j) ^ outcolor1);
-               putcolor(xdots - 1 - i, j, getcolor(xdots - 1 - i, j) ^ outcolor2);
+               putcolor(xdots - 1 - i, j, 
+                  getcolor(xdots - 1 - i, j) ^ outcolor2);
             }
          }
       }
@@ -292,26 +251,9 @@ int savetodisk(char *filename)      /* save-to-disk routine */
 
 int encoder()
 {
-   int i, ydot, xdot, color, outcolor1, outcolor2;
-   int width;
-   int rownum, rowlimit;
-   unsigned int hashentry;
+   int i, width, rowlimit, interrupted;
    BYTE bitsperpixel, x;
-   int entrynum;
    struct fractal_info save_info;
-   unsigned int maxstring;
-
-   max_nextentry=0;
-   ydot = 0;
-
-   /* arbitrary math uses top of extraseg - steal from encoder if necessary */
-   if (bf_save_len)
-   {
-      maxstring = (unsigned int) (0x10000l - (bf_save_len + 2) * 22);
-      maxstring = min((unsigned int) MAXSTRING, maxstring);
-   }
-   else
-      maxstring = (unsigned int) MAXSTRING;
 
    if (initbatch)               /* flush any impending keystrokes */
       while (keypressed())
@@ -340,33 +282,14 @@ int encoder()
    }
 #endif
 
-   clearcode = 1 << (startbits - 1);    /* set clear and end codes */
-   endcode = clearcode + 1;
-
-   outcolor1 = 0;               /* use these colors to show progress */
-   outcolor2 = 1;               /* (this has nothing to do with GIF) */
-   if (colors > 2)
-   {
-      outcolor1 = 2;
-      outcolor2 = 3;
-   }
-   if (((++numsaves) & 1) == 0)
-   {                            /* reverse the colors on alt saves */
-      i = outcolor1;
-      outcolor1 = outcolor2;
-      outcolor2 = i;
-   }
-   outcolor1s = outcolor1;
-   outcolor2s = outcolor2;
-
    if (gif87a_flag == 1)
    {
-      if (fwrite("GIF87a", 6, 1, out) != 1)
+      if (fwrite("GIF87a", 6, 1, g_outfile) != 1)
          goto oops;             /* old GIF Signature */
    }
    else
    {
-      if (fwrite("GIF89a", 6, 1, out) != 1)
+      if (fwrite("GIF89a", 6, 1, g_outfile) != 1)
          goto oops;             /* new GIF Signature */
    }
 
@@ -381,15 +304,15 @@ int encoder()
       rowlimit <<= 1;
       width <<= 1;
    }
-   if (write2(&width, 2, 1, out) != 1)
+   if (write2(&width, 2, 1, g_outfile) != 1)
       goto oops;                /* screen descriptor */
-   if (write2(&ydots, 2, 1, out) != 1)
+   if (write2(&ydots, 2, 1, g_outfile) != 1)
       goto oops;
    x = (BYTE) (128 + ((6 - 1) << 4) + (bitsperpixel - 1));      /* color resolution == 6
                                                                  * bits worth */
-   if (write1(&x, 1, 1, out) != 1)
+   if (write1(&x, 1, 1, g_outfile) != 1)
       goto oops;
-   if (fputc(0, out) != 0)
+   if (fputc(0, g_outfile) != 0)
       goto oops;                /* background color */
    i = 0;
 
@@ -405,7 +328,7 @@ int encoder()
    if (gif87a_flag)
       i = 0;                    /* for some decoders which can't handle
                                  * aspect */
-   if (fputc(i, out) != i)
+   if (fputc(i, g_outfile) != i)
       goto oops;                /* pixel aspect ratio */
 
 #ifndef XFRACT
@@ -413,33 +336,33 @@ int encoder()
    {                            /* write out the 256-color palette */
       if (gotrealdac)
       {                         /* got a DAC - must be a VGA */
-         if (!shftwrite((BYTE *) dacbox, colors))
+         if (!shftwrite((BYTE far *) dacbox, colors))
             goto oops;
 #else
    if (colors > 2)
    {
       if (gotrealdac)
       {                         /* got a DAC - must be a VGA */
-         if (!shftwrite((BYTE *) dacbox, 256))
+         if (!shftwrite((BYTE far *) dacbox, 256))
             goto oops;
 #endif
       }
       else
       {                         /* uh oh - better fake it */
          for (i = 0; i < 256; i += 16)
-            if (!shftwrite(paletteEGA, 16))
+            if (!shftwrite((BYTE far *)paletteEGA, 16))
                goto oops;
       }
    }
    if (colors == 2)
    {                            /* write out the B&W palette */
-      if (!shftwrite(paletteBW, colors))
+      if (!shftwrite((BYTE far *)paletteBW, colors))
          goto oops;
    }
 #ifndef XFRACT
    if (colors == 4)
    {                            /* write out the CGA palette */
-      if (!shftwrite(paletteCGA, colors))
+      if (!shftwrite((BYTE far *)paletteCGA, colors))
          goto oops;
    }
    if (colors == 16)
@@ -451,166 +374,51 @@ int encoder()
       }
       else
       {                         /* no DAC - must be an EGA */
-         if (!shftwrite(paletteEGA, colors))
+         if (!shftwrite((BYTE far *)paletteEGA, colors))
             goto oops;
       }
    }
 #endif
 
-   if (fwrite(",", 1, 1, out) != 1)
+   if (fwrite(",", 1, 1, g_outfile) != 1)
       goto oops;                /* Image Descriptor */
    i = 0;
-   if (write2(&i, 2, 1, out) != 1)
+   if (write2(&i, 2, 1, g_outfile) != 1)
       goto oops;
-   if (write2(&i, 2, 1, out) != 1)
+   if (write2(&i, 2, 1, g_outfile) != 1)
       goto oops;
-   if (write2(&width, 2, 1, out) != 1)
+   if (write2(&width, 2, 1, g_outfile) != 1)
       goto oops;
-   if (write2(&ydots, 2, 1, out) != 1)
+   if (write2(&ydots, 2, 1, g_outfile) != 1)
       goto oops;
-   if (write1(&i, 1, 1, out) != 1)
+   if (write1(&i, 1, 1, g_outfile) != 1)
       goto oops;
 
-   bitsperpixel = (BYTE) (startbits - 1);       /* raster data starts here */
-   if (write1(&bitsperpixel, 1, 1, out) != 1)
+   bitsperpixel = (BYTE) (startbits - 1);
+
+   if (write1(&bitsperpixel, 1, 1, g_outfile) != 1)
       goto oops;
 
-   codebits = startbits;        /* start encoding */
+   interrupted = compress(rowlimit);
 
-   if (!raster(9999))
-      goto oops;                /* initialize the raster routine */
+   if(ferror(g_outfile))
+      goto oops;
 
-   if (!inittable())
-      goto oops;                /* initialize the LZW tables */
-
-   for (rownum = 0; rownum < ydots; rownum++
-#ifdef WINFRACT
-        ,UpdateStatusBox(rownum, ydots)
-#endif
-      )
-   {                            /* scan through the dots */
-      for (ydot = rownum; ydot < rowlimit; ydot += ydots)
-      {
-         for (xdot = 0; xdot < xdots; xdot++)
-         {
-            if (save16bit == 0 || ydot < ydots)
-               color = getcolor(xdot, ydot);
-            else
-               color = readdisk(xdot + sxoffs, ydot + syoffs);
-            teststring[0] = (BYTE)++ lentest;
-            teststring[lentest] = (BYTE) color;
-            if (lentest == 1)
-            {                   /* root entry? */
-               lastentry = color;
-               continue;
-            }
-            if (lentest == 2)   /* init   the hash code */
-               hashcode = 301 * (teststring[1] + 1);
-            hashcode *= (color + lentest);      /* update the hash code */
-            hashentry = ++hashcode % MAXENTRY;
-            for (i = 0; i < MAXENTRY; i++)
-            {
-               if (++hashentry >= MAXENTRY)
-                  hashentry = 0;
-               if (cmpextra(strlocn[hashentry] + sizeof(int),
-                            (char *) teststring, lentest + 1) == 0)
-                  break;
-               if (strlocn[hashentry] == 0)
-                  i = MAXENTRY;
-            }
-            /* found an entry and string length isn't too bad */
-            if (strlocn[hashentry] != 0 &&
-                lentest < MAXTEST - 1 - (int)sizeof(int) - lzw[0])
-            {
-               fromextra(strlocn[hashentry], (char *) &entrynum, sizeof(int));
-               lastentry = entrynum;
-               continue;
-            }
-            if (!raster(lastentry))
-               goto oops;       /* write entry */
-            numentries++;       /* act like you added one, anyway */
-            if (strlocn[hashentry] == 0)
-            {                   /* add new string, if any */
-               entrynum = numentries + endcode;
-               strlocn[hashentry] = nextentry;
-               toextra(nextentry, (char *) &entrynum, sizeof(int));
-               toextra(nextentry + sizeof(int),
-                       (char *) teststring, lentest + 1);
-               nextentry += lentest + 1 + sizeof(int);
-               if((long)nextentry > max_nextentry)
-                  max_nextentry = nextentry;
-               numrealentries++;
-            }
-            teststring[0] = 1;  /* reset current entry */
-            teststring[1] = (BYTE) color;
-            lentest = 1;
-            lastentry = color;
-
-            if ((numentries + endcode) == (1 << codebits))
-               codebits++;      /* use longer encoding */
-
-            if (numentries + endcode > 4093 ||  /* out of room? */
-                numrealentries > (MAXENTRY * 2) / 3 ||
-                nextentry > maxstring - MAXTEST - 1 - 2 * sizeof(int) - lzw[1])
-            {
-               if (!raster(lastentry))
-                  goto oops;    /* flush & restart */
-               if (!inittable())
-                  goto oops;
-            }
-         }
-         if (dotmode != 11      /* supress this on disk-video */
-             && active_system == 0      /* and in Windows version     */
-             && ydot == rownum)
-         {
-            if ((ydot & 4) == 0)
-            {
-               if (++outcolor1 >= colors)
-                  outcolor1 = 0;
-               if (++outcolor2 >= colors)
-                  outcolor2 = 0;
-            }
-            for (i = 0; 250 * i < xdots; i++)
-            {                   /* display vert status bars */
-               /* (this is NOT GIF-related)  */
-               /* PB Changed following code to xor color, so that image can
-                * be restored at end and resumed putcolor(
-                * i,ydot,outcolor1); putcolor(xdots-1-i,ydot,outcolor2); */
-               putcolor(i, ydot, getcolor(i, ydot) ^ outcolor1);
-               putcolor(xdots - 1 - i, ydot, getcolor(xdots - 1 - i, ydot) ^ outcolor2);
-            }
-            last_colorbar = ydot;
-         }
-#ifdef WINFRACT
-         keypressed();
-         if (OperCancelled)
-#else
-         if (keypressed())      /* keyboard hit - bail out */
-#endif
-            ydot = rownum = 9999;
-      }
-   }
-
-   if (!raster(lastentry))
-      goto oops;                /* tidy up - dump the last code */
-
-   if (!raster(endcode))
-      goto oops;                /* finish the map */
-
-   if (fputc(0, out) != 0)
-      goto oops;                /* raster data ends here */
+   if (fputc(0, g_outfile) != 0)
+      goto oops;
 
    if (gif87a_flag == 0)
    {                            /* store non-standard fractal info */
       /* loadfile.c has notes about extension block structure */
-      if (ydot >= 9999)
+      if (interrupted)
          save_info.calc_status = 0;     /* partial save is not resumable */
       save_info.tot_extend_len = 0;
-      if (resume_info != NULL && save_info.calc_status == 2)
+      if (resume_info != 0 && save_info.calc_status == 2)
       {
          /* resume info block, 002 */
          save_info.tot_extend_len += extend_blk_len(resume_len);
-         if (!put_extend_blk(2, resume_len, resume_info))
+         MoveFromMemory((BYTE *)block,(U16)1,(long)resume_len,0,resume_info);
+         if (!put_extend_blk(2, resume_len, (char far *)block))
             goto oops;
       }
       if (save_info.fractal_type == FORMULA || save_info.fractal_type == FFORMULA)
@@ -626,7 +434,6 @@ int encoder()
 #ifdef XFRACT
          fix_ranges(ranges, rangeslen, 0);
          put_extend_blk(4, rangeslen * 2, (char far *) ranges);
-         fix_ranges(ranges, rangeslen, 0);
 #else
          if (!put_extend_blk(4, rangeslen * 2, (char far *) ranges))
             goto oops;
@@ -642,6 +449,68 @@ int encoder()
             goto oops;
       }
 
+      /* Extended parameters block 006 */
+      if (evolving&1)
+      {
+          struct evolution_info esave_info;
+          int i;
+          struct evolution_info resume_e_info;
+          GENEBASE gene[NUMGENES];
+          MoveFromMemory((BYTE *)&gene, (U16)sizeof(gene), 1L, 0L, gene_handle);
+          if (evolve_handle == 0) {
+             esave_info.paramrangex     = paramrangex;
+             esave_info.paramrangey     = paramrangey;
+             esave_info.opx             = opx;
+             esave_info.opy             = opy;
+             esave_info.odpx            = (short)odpx;
+             esave_info.odpy            = (short)odpy;
+             esave_info.px              = (short)px;
+             esave_info.py              = (short)py;
+             esave_info.sxoffs          = (short)sxoffs;
+             esave_info.syoffs          = (short)syoffs;
+             esave_info.xdots           = (short)xdots;
+             esave_info.ydots           = (short)ydots;
+             esave_info.gridsz          = (short)gridsz;
+             esave_info.evolving        = (short)evolving;
+             esave_info.this_gen_rseed  = (unsigned short)this_gen_rseed;
+             esave_info.fiddlefactor    = fiddlefactor;
+             esave_info.ecount          = (short) (gridsz * gridsz); /* flag for done */
+          }
+          else { /* we will need the resuming information */
+             MoveFromMemory((BYTE *)&resume_e_info,(U16)sizeof(resume_e_info),1L,0L,evolve_handle);
+             esave_info.paramrangex     = resume_e_info.paramrangex;
+             esave_info.paramrangey     = resume_e_info.paramrangey;
+             esave_info.opx             = resume_e_info.opx;
+             esave_info.opy             = resume_e_info.opy;
+             esave_info.odpx            = (short)resume_e_info.odpx;
+             esave_info.odpy            = (short)resume_e_info.odpy;
+             esave_info.px              = (short)resume_e_info.px;
+             esave_info.py              = (short)resume_e_info.py;
+             esave_info.sxoffs          = (short)resume_e_info.sxoffs;
+             esave_info.syoffs          = (short)resume_e_info.syoffs;
+             esave_info.xdots           = (short)resume_e_info.xdots;
+             esave_info.ydots           = (short)resume_e_info.ydots;
+             esave_info.gridsz          = (short)resume_e_info.gridsz;
+             esave_info.evolving        = (short)resume_e_info.evolving;
+             esave_info.this_gen_rseed  = (unsigned short)resume_e_info.this_gen_rseed;
+             esave_info.fiddlefactor    = resume_e_info.fiddlefactor;
+             esave_info.ecount          = resume_e_info.ecount;
+          }
+          for (i = 0; i < NUMGENES; i++)
+             esave_info.mutate[i] = (short)gene[i].mutate;
+
+          for (i = 0; i < sizeof(esave_info.future) / sizeof(short); i++)
+             esave_info.future[i] = 0;
+
+          /* some XFRACT logic for the doubles needed here */
+#ifdef XFRACT
+          decode_evolver_info(&esave_info, 0);
+#endif
+          /* evolution info block, 006 */
+          put_extend_blk(6, sizeof(esave_info), (char far *) &esave_info);
+          save_info.tot_extend_len += extend_blk_len(sizeof(esave_info));
+      }
+
       /* main and last block, 001 */
       save_info.tot_extend_len += extend_blk_len(FRACTAL_INFO_SIZE);
 #ifdef XFRACT
@@ -653,22 +522,22 @@ int encoder()
       }
    }
 
-   if (fwrite(";", 1, 1, out) != 1)
+   if (fwrite(";", 1, 1, g_outfile) != 1)
       goto oops;                /* GIF Terminator */
 
-   return ((ydot < 9999) ? 0 : 1);
+   return (interrupted);
 
- oops:
+oops:
    {
       static FCODE msg[] = {"Error Writing to disk (Disk full?)"};
-      fflush(out);
+      fflush(g_outfile);
       stopmsg(0,msg);
       return 1;
    }
 }
 
 /* shift IBM colors to GIF */
-static int _fastcall shftwrite(BYTE * color, int numcolors)
+static int _fastcall shftwrite(BYTE far * color, int numcolors)
 {
    BYTE thiscolor;
    int i, j;
@@ -678,81 +547,12 @@ static int _fastcall shftwrite(BYTE * color, int numcolors)
          thiscolor = color[3 * i + j];
          thiscolor = (BYTE) (thiscolor << 2);
          thiscolor = (BYTE) (thiscolor + (BYTE) (thiscolor >> 6));
-         if (fputc(thiscolor, out) != (int) thiscolor)
+         if (fputc(thiscolor, g_outfile) != (int) thiscolor)
             return (0);
       }
    return (1);
 }
 
-static int inittable()                     /* routine to init tables */
-{
-   int i;
-
-   if (!raster(clearcode))
-      return (0);               /* signal that table is initialized */
-
-   numentries = 0;              /* initialize the table */
-   numrealentries = 0;
-   nextentry = 1;
-   lentest = 0;
-   codebits = startbits;
-
-   toextra(0, "\0", 1);         /* clear the hash entries */
-   for (i = 0; i < MAXENTRY; i++)
-      strlocn[i] = 0;
-
-   return (1);
-}
-
-/* routine to block and output codes */
-static int _fastcall raster(unsigned int code)
-{
-   unsigned int icode, i, j;
-
-   if (code == 9999)
-   {                            /* special start-up signal */
-      bytecount = 0;
-      bitcount = 0;
-      for (i = 0; i < 266; i++)
-         block[i] = 0;
-      return (1);
-   }
-
-   icode = code << bitcount;    /* update the bit string */
-   block[bytecount] |= (icode & 255);
-   block[bytecount + 1] |= ((icode >> 8) & 255);
-   icode = (code >> 8) << bitcount;
-   block[bytecount + 2] |= ((icode >> 8) & 255);
-   bitcount += codebits;
-   while (bitcount >= 8)
-   {                            /* locate next starting point */
-      bitcount -= 8;
-      bytecount++;
-   }
-
-   if (bytecount > 250 || code == (unsigned int) endcode)
-   {                            /* time to write a block */
-      if (code == (unsigned int) endcode)
-         while (bitcount > 0)
-         {                      /* if EOF, find the real end */
-            bitcount -= 8;
-            bytecount++;
-         }
-      i = bytecount;
-      blockcount = (BYTE) i;
-      if (write1(&blockcount, 1, 1, out) != 1)
-         return (0);            /* write the block */
-      if (fwrite(block, i, 1, out) != 1)
-         return (0);
-      bytecount = 0;            /* now re-start the block */
-      for (j = 0; j < 5; j++)   /* (may have leftover bits) */
-         block[j] = block[j + i];
-      for (j = 5; j < 266; j++)
-         block[j] = 0;
-   }
-   return (1);
-}
-
 static int _fastcall extend_blk_len(int datalen)
 {
    return (datalen + (datalen + 254) / 255 + 15);
@@ -765,18 +565,18 @@ static int _fastcall put_extend_blk(int block_id, int block_len, char far * bloc
    char header[15];
    strcpy(header, "!\377\013fractint");
    sprintf(&header[11], "%03u", block_id);
-   if (fwrite(header, 14, 1, out) != 1)
+   if (fwrite(header, 14, 1, g_outfile) != 1)
       return (0);
    i = (block_len + 254) / 255;
    while (--i >= 0)
    {
       block_len -= (j = min(block_len, 255));
-      if (fputc(j, out) != j)
+      if (fputc(j, g_outfile) != j)
          return (0);
       while (--j >= 0)
-         fputc(*(block_data++), out);
+         fputc(*(block_data++), g_outfile);
    }
-   if (fputc(0, out) != 0)
+   if (fputc(0, g_outfile) != 0)
       return (0);
    return (1);
 }
@@ -793,12 +593,16 @@ static int _fastcall store_item_name(char *nameptr)
       fsave_info.uses_p1 = (short) uses_p1;
       fsave_info.uses_p2 = (short) uses_p2;
       fsave_info.uses_p3 = (short) uses_p3;
+      fsave_info.uses_ismand = (short) uses_ismand;
+      fsave_info.ismand = (short) ismand;
    }
    else
    {
       fsave_info.uses_p1 = 0;
       fsave_info.uses_p2 = 0;
       fsave_info.uses_p3 = 0;
+      fsave_info.uses_ismand = 0;
+      fsave_info.ismand = 0;
    }
    for (i = 0; i < sizeof(fsave_info.future) / sizeof(short); i++)
       fsave_info.future[i] = 0;
@@ -814,7 +618,7 @@ static void _fastcall setup_save_info(struct fractal_info far * save_info)
       maxfn = 0;
    /* set save parameters in save structure */
    far_strcpy(save_info->info_id, INFO_ID);
-   save_info->version = 12;     /* file version, independent of system */
+   save_info->version = 13;     /* file version, independent of system */
    /* increment this EVERY time the fractal_info structure changes */
 
    if (maxit <= SHRT_MAX)
@@ -950,7 +754,373 @@ static void _fastcall setup_save_info(struct fractal_info far * save_info)
    save_info->dinvert[2] = inversion[2];
    save_info->logcalc = (short) Log_Fly_Calc;
    save_info->stoppass = (short) stoppass;
+   save_info->quick_calc = (short) quick_calc;
+   save_info->closeprox = closeprox;
    for (i = 0; i < sizeof(save_info->future) / sizeof(short); i++)
       save_info->future[i] = 0;
+      
 }
 
+/***************************************************************************
+ *
+ *  GIFENCOD.C       - GIF Image compression routines
+ *
+ *  Lempel-Ziv compression based on 'compress'.  GIF modifications by
+ *  David Rowley (mgardi@watdcsu.waterloo.edu). 
+ *  Thoroughly massaged by the Stone Soup team for Fractint's purposes.
+ * 
+ ***************************************************************************/
+
+#define BITSF   12
+#define HSIZE  5003            /* 80% occupancy */
+
+/*
+ *
+ * GIF Image compression - modified 'compress'
+ *
+ * Based on: compress.c - File compression ala IEEE Computer, June 1984.
+ *
+ * By Authors:  Spencer W. Thomas       (decvax!harpo!utah-cs!utah-gr!thomas)
+ *              Jim McKie               (decvax!mcvax!jim)
+ *              Steve Davies            (decvax!vax135!petsd!peora!srd)
+ *              Ken Turkowski           (decvax!decwrl!turtlevax!ken)
+ *              James A. Woods          (decvax!ihnp4!ames!jaw)
+ *              Joe Orost               (decvax!vax135!petsd!joe)
+ *
+ */
+
+/* prototypes */
+
+static void _fastcall output(int code);
+static void _fastcall char_out(int c);
+static void _fastcall flush_char(void);
+static void _fastcall cl_block(void);
+
+static int n_bits;                        /* number of bits/code */
+static int maxbits = BITSF;                /* user settable max # bits/code */
+static int maxcode;                  /* maximum code, given n_bits */
+static int maxmaxcode = (int)1 << BITSF; /* should NEVER generate this code */
+# define MAXCODE(n_bits)        (((int) 1 << (n_bits)) - 1)
+
+#ifdef XFRACT
+unsigned int strlocn[10240];
+BYTE block[4096];
+#endif
+
+static long far *htab;
+static unsigned short *codetab = (unsigned short *)strlocn;
+
+/*
+ * To save much memory, we overlay the table used by compress() with those
+ * used by decompress().  The tab_prefix table is the same size and type
+ * as the codetab.  The tab_suffix table needs 2**BITSF characters.  We
+ * get this from the beginning of htab.  The output stack uses the rest
+ * of htab, and contains characters.  There is plenty of room for any
+ * possible stack (stack used to be 8000 characters).
+ */
+
+#define tab_prefixof(i)   codetab[i]
+#define tab_suffixof(i)   ((char_type far *)(htab))[i]
+#define de_stack          ((char_type far *)&tab_suffixof((int)1<<BITSF))
+
+static int free_ent;                  /* first unused entry */
+
+/*
+ * block compression parameters -- after all codes are used up,
+ * and compression rate changes, start over.
+ */
+static int clear_flg = 0;
+
+/*
+ * compress stdin to stdout
+ *
+ * Algorithm:  use open addressing double hashing (no chaining) on the 
+ * prefix code / next character combination.  We do a variant of Knuth's
+ * algorithm D (vol. 3, sec. 6.4) along with G. Knott's relatively-prime
+ * secondary probe.  Here, the modular division first probe is gives way
+ * to a faster exclusive-or manipulation.  Also do block compression with
+ * an adaptive reset, whereby the code table is cleared when the compression
+ * ratio decreases, but after the table fills.  The variable-length output
+ * codes are re-sized at this point, and a special CLEAR code is generated
+ * for the decompressor.  Late addition:  construct the table according to
+ * file size for noticeable speed improvement on small files.  Please direct
+ * questions about this implementation to ames!jaw.
+ */
+
+static int ClearCode;
+static int EOFCode;
+static int a_count; /* Number of characters so far in this 'packet' */
+static unsigned long cur_accum = 0;
+static int  cur_bits = 0;
+
+/*
+ * Define the storage for the packet accumulator
+ */
+static char *accum; /* 256 bytes */
+
+static int compress(int rowlimit)
+{
+   int outcolor1, outcolor2;
+   long fcode;
+   int i = 0;
+   int ent;
+   int disp;
+   int hsize_reg;
+   int hshift;
+   int ydot, xdot, color;
+   int rownum;
+   int in_count = 0;
+   int interrupted = 0;
+   int tempkey;
+   char accum_stack[256];
+   accum = accum_stack;
+   htab = (long far *)MK_FP(extraseg,0);
+   
+   outcolor1 = 0;               /* use these colors to show progress */
+   outcolor2 = 1;               /* (this has nothing to do with GIF) */
+
+   if (colors > 2)
+   {
+      outcolor1 = 2;
+      outcolor2 = 3;
+   }
+   if (((++numsaves) & 1) == 0)
+   {                            /* reverse the colors on alt saves */
+      i = outcolor1;
+      outcolor1 = outcolor2;
+      outcolor2 = i;
+   }
+   outcolor1s = outcolor1;
+   outcolor2s = outcolor2;
+
+   /*
+    * Set up the necessary values
+    */
+   cur_accum = 0;
+   cur_bits = 0;
+   clear_flg = 0;
+   ydot = 0;
+   ent = 0;
+   maxcode = MAXCODE(n_bits = startbits);
+
+   ClearCode = (1 << (startbits - 1));
+   EOFCode = ClearCode + 1;
+   free_ent = ClearCode + 2;
+
+   a_count = 0;
+   hshift = 0;
+   for (fcode = (long) HSIZE;  fcode < 65536L; fcode *= 2L)
+       hshift++;
+   hshift = 8 - hshift;                /* set hash code range bound */
+
+   far_memset(htab,0xff,(unsigned)HSIZE*sizeof(long));
+   hsize_reg = HSIZE;
+
+   output((int)ClearCode);
+
+   for (rownum = 0; rownum < ydots; rownum++)
+   {                            /* scan through the dots */
+      for (ydot = rownum; ydot < rowlimit; ydot += ydots)
+      {
+         for (xdot = 0; xdot < xdots; xdot++)
+         {
+            if (save16bit == 0 || ydot < ydots)
+               color = getcolor(xdot, ydot);
+            else
+               color = readdisk(xdot + sxoffs, ydot + syoffs);
+            if(in_count == 0)
+            {
+               in_count = 1;
+               ent = color;
+               continue;
+            }
+            fcode = (long) (((long) color << maxbits) + ent);
+            i = (((int)color << hshift) ^ ent);    /* xor hashing */
+     
+            if (htab[i] == fcode) 
+            {
+                ent = codetab[i];
+                continue;
+            } else if ((long)htab[i] < 0)      /* empty slot */
+                goto nomatch;
+            disp = hsize_reg - i;           /* secondary hash (after G. Knott) */
+            if (i == 0)
+                disp = 1;
+probe:
+            if ((i -= disp) < 0)
+               i += hsize_reg;
+     
+            if (htab[i] == fcode) 
+            {
+                ent = codetab[i];
+                continue;
+            }
+            if ((long)htab[i] > 0) 
+                goto probe;
+nomatch:
+            output ((int) ent);
+            ent = color;
+            if (free_ent < maxmaxcode) 
+            {
+                /* code -> hashtable */
+                codetab[i] = (unsigned short)free_ent++; 
+                htab[i] = fcode;
+            } 
+            else
+              cl_block();
+         } /* end for xdot */
+         if (dotmode != 11      /* supress this on disk-video */
+             && active_system == 0      /* and in Windows version     */
+             && ydot == rownum)
+         {
+            if ((ydot & 4) == 0)
+            {
+               if (++outcolor1 >= colors)
+                  outcolor1 = 0;
+               if (++outcolor2 >= colors)
+                  outcolor2 = 0;
+            }
+            for (i = 0; 250 * i < xdots; i++)
+            {  /* display vert status bars */
+               /* (this is NOT GIF-related)  */
+               putcolor(i, ydot, getcolor(i, ydot) ^ outcolor1);
+               putcolor(xdots - 1 - i, ydot, 
+                  getcolor(xdots - 1 - i, ydot) ^ outcolor2);
+            }
+            last_colorbar = ydot;
+         } /* end if dotmode != 11 */
+         tempkey = keypressed();
+         if (tempkey && (tempkey != (int)'s'))  /* keyboard hit - bail out */
+         {
+            interrupted = 1;
+            rownum = ydots;
+            break;
+         }
+         if (tempkey == (int)'s')
+            getakey();   /* eat the keystroke */
+      } /* end for ydot */
+   } /* end for rownum */
+   
+   /*
+    * Put out the final code.
+    */
+   output((int)ent);
+   output((int) EOFCode);
+   return (interrupted);
+}
+
+/*****************************************************************
+ * TAG(output)
+ *
+ * Output the given code.
+ * Inputs:
+ *      code:   A n_bits-bit integer.  If == -1, then EOF.  This assumes
+ *              that n_bits =< (long)wordsize - 1.
+ * Outputs:
+ *      Outputs code to the file.
+ * Assumptions:
+ *      Chars are 8 bits long.
+ * Algorithm:
+ *      Maintain a BITSF character long buffer (so that 8 codes will
+ * fit in it exactly).  Use the VAX insv instruction to insert each
+ * code in turn.  When the buffer fills up empty it and start over.
+ */
+
+
+static void _fastcall output(int code)
+{
+   static ULFCODE masks[] = 
+      { 0x0000, 0x0001, 0x0003, 0x0007, 0x000F,
+                0x001F, 0x003F, 0x007F, 0x00FF,
+                0x01FF, 0x03FF, 0x07FF, 0x0FFF,
+                0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF };
+                
+   cur_accum &= masks[ cur_bits ];
+
+   if(cur_bits > 0)
+      cur_accum |= ((long)code << cur_bits);
+   else
+      cur_accum = code;
+   
+   cur_bits += n_bits;
+
+   while(cur_bits >= 8) 
+   {
+      char_out((unsigned int)(cur_accum & 0xff));
+      cur_accum >>= 8;
+      cur_bits -= 8;
+   }
+
+   /*
+    * If the next entry is going to be too big for the code size,
+    * then increase it, if possible.
+    */
+   if (free_ent > maxcode || clear_flg) 
+   {
+      if(clear_flg) 
+      {
+         maxcode = MAXCODE (n_bits = startbits);
+         clear_flg = 0;
+      
+      } 
+      else 
+      {
+         n_bits++;
+         if (n_bits == maxbits)
+            maxcode = maxmaxcode;
+         else
+            maxcode = MAXCODE(n_bits);
+      }
+   }
+   
+   if(code == EOFCode) 
+   {
+      /*
+       * At EOF, write the rest of the buffer.
+       */
+       while(cur_bits > 0) 
+       {
+          char_out((unsigned int)(cur_accum & 0xff));
+          cur_accum >>= 8;
+          cur_bits -= 8;
+       }
+
+       flush_char();
+  
+       fflush(g_outfile);
+   }
+}
+
+/*
+ * Clear out the hash table
+ */
+static void _fastcall cl_block(void)             /* table clear for block compress */
+{
+        far_memset(htab,0xff,(unsigned)HSIZE*sizeof(long));
+        free_ent = ClearCode + 2;
+        clear_flg = 1;
+        output((int)ClearCode);
+}
+
+/*
+ * Add a character to the end of the current packet, and if it is 254
+ * characters, flush the packet to disk.
+ */
+static void _fastcall char_out(int c)
+{
+   accum[ a_count++ ] = (char)c;
+   if(a_count >= 254) 
+      flush_char();
+}
+
+/*
+ * Flush the packet to disk, and reset the accumulator
+ */
+static void _fastcall flush_char(void)
+{
+   if(a_count > 0) {
+      fputc(a_count, g_outfile);
+      fwrite(accum, 1, a_count, g_outfile);
+      a_count = 0;
+   }
+}  
