@@ -28,6 +28,7 @@ extern double                AutoStereo_width;
 extern BYTE                  back_color[];
 extern int                   badconfig;
 extern int                   bad_code_count;
+extern int                   bad_outside;
 extern int                   bad_value;
 extern long                  bailout;
 extern enum bailouts         bailoutest;
@@ -61,6 +62,7 @@ extern char                  calibrate;
 extern int                   checkcurdir;
 extern long                  cimag;
 extern double                closenuff;
+extern double                closeprox;
 extern _CMPLX                coefficient;
 extern int                   col;
 extern int                   color;
@@ -74,7 +76,7 @@ extern int                   color_bright;
 extern int                   color_dark;
 extern int                   color_medium;
 extern char                  CommandComment[4][MAXCMT];
-extern char                  CommandFile[80];
+extern char                  CommandFile[FILE_MAX_PATH];
 extern char                  CommandName[ITEMNAMELEN + 1];
 extern int                   comparegif;
 extern long                  con;
@@ -97,8 +99,6 @@ extern BYTE                  decoderline[];
 extern int                   decomp[];
 extern int                   degree;
 extern long                  delmin;
-extern double                deltaX;
-extern double                deltaY;
 extern long                  delx2;
 extern long                  delx;
 extern LDBL                  delxx2;
@@ -109,7 +109,6 @@ extern LDBL                  delyy2;
 extern LDBL                  delyy;
 extern float                 depthfp;
 extern int                   disk16bit;
-extern char                  diskfilename[];
 extern int                   diskflag;
 extern int                   diskisactive;
 extern int                   diskvideo;
@@ -122,17 +121,24 @@ extern int                   dither_flag;
 extern char                  dontreadcolor;
 extern int                   dotmode;
 extern int                   doublecaution;
+extern double                dpx;
+extern double                dpy;
 extern BYTE                  dstack[];
+extern U16                   dv_handle;
 extern double far *          dx0;
 extern double far *          dx1;
+extern double (_fastcall *   dxpixel)(void); /* set in FRACTALS.C */
 extern double                dxsize;
 extern double far *          dy0;
 extern double far *          dy1;
+extern double (_fastcall *   dypixel)(void); /* set in FRACTALS.C */
 extern double                dysize;
 extern int                   EPSFileType;
 extern int                   escape_exit;
 extern BYTE                  exitmode;
 extern SEGTYPE               extraseg;
+extern int                   evolving;
+extern U16                   evolve_handle;
 extern int                   eyeseparation;
 extern float                 eyesfp;
 extern int                   fastrestore;
@@ -140,6 +146,9 @@ extern long                  FgHalf;
 extern double                fgLimit;
 extern long                  FgOne;
 extern long                  FgTwo;
+extern int                   gridsz;
+extern double                fiddlefactor;
+extern double                fiddle_reduction;
 extern float                 fileaspectratio;
 extern int                   filecolors;
 extern int                   filetype;
@@ -155,6 +164,12 @@ extern char                  floatflag;
 extern double                floatmax;
 extern double                floatmin;
 extern _CMPLX *              floatparm;
+extern int                   fm_attack;
+extern int                   fm_decay;
+extern int                   fm_release;
+extern int                   fm_sustain;
+extern int                   fm_wavetype;
+extern int                   fm_vol; /*volume of OPL-3 soundcard output*/
 extern int                   forcesymmetry;
 extern char                  FormFileName[];
 extern char                  FormName[];
@@ -169,6 +184,7 @@ extern double                f_at_rad;
 extern double                f_radius;
 extern double                f_xcenter;
 extern double                f_ycenter;
+extern U16                   gene_handle;
 extern int                   get_corners(void);
 extern int                   gif87a_flag;
 extern char                  gifmask[];
@@ -184,12 +200,14 @@ extern int                   haze;
 extern unsigned int          height;
 extern float                 heightfp;
 extern int                   helpmode;
-extern HISTORY far *         history;
+extern int                   hi_atten;
+extern U16                   history;
 extern char                  IFSFileName[];
 extern char                  IFSName[];
 extern float far *           ifs_defn;
 extern int                   ifs_type;
-extern int                   iit;
+extern int                   imgboxcount;
+extern U16                   imgboxhandle;
 extern char                  image_map;
 extern int                   init3d[20];
 extern _CMPLX                init;
@@ -204,6 +222,7 @@ extern int                   integerfractal;
 extern double                inversion[];
 extern int                   invert;
 extern int                   istruecolor;
+extern short                 ismand;
 extern int                   ixstart;
 extern int                   ixstop;
 extern int                   iystart;
@@ -222,6 +241,7 @@ extern int                   lastorbittype;
 extern _LCMPLX               lattr[];
 extern long                  lclosenuff;
 extern _LCMPLX               lcoefficient;
+extern int                   ldcheck;
 extern char                  LFileName[];
 extern char                  light_name[];
 extern BYTE *                line_buff;
@@ -251,12 +271,13 @@ extern long                  ltempsqry;
 extern _LCMPLX               ltmp;
 extern long far *            lx0;
 extern long far *            lx1;
+extern long (_fastcall *     lxpixel)(void); /* set in FRACTALS.C */
 extern long far *            ly0;
 extern long far *            ly1;
+extern long (_fastcall *     lypixel)(void); /* set in FRACTALS.C */
 extern int                   lzw[2];
 extern long                  l_at_rad;
 extern MATRIX                m;
-extern int                   made_dsktemp;
 extern double                magnitude;
 extern enum Major            major_method;
 extern BYTE far *            mapdacbox;
@@ -268,6 +289,7 @@ extern int                   maxcolor;
 extern long                  maxct;
 extern char                  maxfn;
 extern long                  maxit;
+extern int                   maxlinelength;
 extern long                  MaxLTSize;
 extern unsigned              Max_Args;
 extern unsigned              Max_Ops;
@@ -275,8 +297,11 @@ extern long                  maxptr;
 extern int                   max_colors;
 extern int                   max_kbdcount;
 extern int                   maxhistory;
+extern int                   max_rhombus_depth;
 extern int                   minbox;
 extern enum Minor            minor_method;
+extern int                   minstack;
+extern int                   minstackavail;
 extern int                   mode7text;
 extern MOREPARAMS            moreparams[];
 extern struct MP             mpAp1deg;
@@ -297,6 +322,10 @@ extern double                mymaxfp;
 extern double                myminfp;
 extern int                   name_stack_ptr;
 extern _CMPLX                new;
+extern char                  newodpx;
+extern char                  newodpy;
+extern double                newopx;
+extern double                newopy;
 extern int                   neworbittype;
 extern int                   no_sub_images;
 extern int                   no_mag_calc;
@@ -311,11 +340,19 @@ extern int                   oktoprint;
 extern _CMPLX                old;
 extern long                  oldcoloriter;
 extern BYTE                  olddacbox[256][3];
+extern U16                   oldhistory_handle;
 extern int                   old_demm_colors;
+extern char                  old_stdcalcmode;
+extern char                  odpx;
+extern char                  odpy;
+extern double                opx;
+extern double                opy;
 extern int                   orbitsave;
 extern int                   orbit_color;
 extern int                   orbit_delay;
 extern int                   orbit_ptr;
+extern char                  orgfrmdir[];
+extern int                   orgfrmsearch;
 extern float                 originfp;
 extern int (*                outln) (BYTE *, int);
 extern void (*               outln_cleanup) (void);
@@ -324,6 +361,9 @@ extern int                   overflow;
 extern int                   overlay3d;
 extern char                  overwrite;
 extern double                param[];
+extern double                paramrangex;
+extern double                paramrangey;
+extern double                parmzoom;
 extern _CMPLX                parm2;
 extern _CMPLX                parm;
 extern int                   passes;
@@ -336,6 +376,7 @@ extern double                plotmx1;
 extern double                plotmx2;
 extern double                plotmy1;
 extern double                plotmy2;
+extern int                   polyphony;
 extern unsigned              posp;
 extern int                   pot16bit;
 extern int                   potflag;
@@ -345,6 +386,8 @@ extern U16                   prefix[];
 #endif
 extern char                  preview;
 extern int                   previewfactor;
+extern int                   px;
+extern int                   py;
 extern int                   Printer_BAngle;
 extern int                   Printer_BFrequency;
 extern int                   Printer_BStyle;
@@ -366,6 +409,8 @@ extern int                   Printer_Titleblock;
 extern int                   Printer_Type;
 extern char                  PrintName[];
 extern int                   Print_To_File;
+extern U16                   prmboxhandle;
+extern int                   prmboxcount;
 extern int                   pseudox;
 extern int                   pseudoy;
 extern void (_fastcall *     putcolor)(int,int,int);
@@ -374,6 +419,7 @@ extern double                qc;
 extern double                qci;
 extern double                qcj;
 extern double                qck;
+extern int                   quick_calc;
 extern int                   RANDOMIZE;
 extern int far *             ranges;
 extern int                   rangeslen;
@@ -389,11 +435,12 @@ extern int                   red_crop_right;
 extern int                   release;
 extern int                   resave_flag;
 extern int                   reset_periodicity;
-extern char far *            resume_info;
+extern U16                   resume_info;
 extern int                   resume_len;
 extern int                   resuming;
 extern int                   rflag;
 extern char                  rlebuf[];
+extern int                   rhombus_stack[];
 extern int                   root;
 extern _CMPLX *              roots;
 extern int                   rotate_hi;
@@ -412,6 +459,7 @@ extern long                  saveticks;
 extern int far *             save_orbit;
 extern int                   save_release;
 extern int                   save_system;
+extern int                   scale_map[];
 extern float                 screenaspect;
 extern char                  scrnfile[];
 extern struct SearchPath     searchfor;
@@ -427,6 +475,7 @@ extern short                 skipydots;
 extern int                   slides;
 extern int                   Slope;
 extern int                   soundflag;
+extern int                   sound_rollover;
 extern char                  speed_prompt[];
 extern void (_fastcall*      standardplot)(int,int,int);
 extern char                  start_showorbit;
@@ -467,12 +516,15 @@ extern char                  s_asin[];
 extern char                  s_asinh[];
 extern char                  s_atan[];
 extern char                  s_atanh[];
+extern char                  s_attack[];
+extern char                  s_atten[];
 extern char                  s_autokeyname[];
 extern char                  s_autokey[];
 extern char                  s_background[];
 extern char                  s_bailoutest[];
 extern char                  s_bailout[];
 extern char                  s_batch[];
+extern char                  s_beep[];
 extern char                  s_biomorph[];
 extern char                  s_biospalette[];
 extern char                  s_bof60[];
@@ -506,6 +558,7 @@ extern char                  s_cyclelimit[];
 extern char                  s_cyclerange[];
 extern char                  s_debugflag[];
 extern char                  s_debug[];
+extern char                  s_decay[];
 extern char                  s_decomp[];
 extern char                  s_distest[];
 extern char                  s_dither[];
@@ -523,6 +576,7 @@ extern char                  s_finattract[];
 extern char                  s_flip[];
 extern char                  s_float[];
 extern char                  s_floor[];
+extern char                  s_fmod[];
 extern char                  s_fn1[];
 extern char                  s_fn2[];
 extern char                  s_fn3[];
@@ -538,16 +592,17 @@ extern char                  s_halftone[];
 extern char                  s_haze[];
 extern char                  s_hertz[];
 extern char                  s_hgc[];
+extern char                  s_high[];
 extern char                  s_ident[];
 extern char                  s_ifs3d[];
 extern char                  s_ifsfile[];
 extern char                  s_ifs[];
-extern char                  s_iit[];
 extern char                  s_imag[];
 extern char                  s_initorbit[];
 extern char                  s_inside[];
 extern char                  s_interocular[];
 extern char                  s_invert[];
+extern char                  s_ismand[];
 extern char                  s_iterincr[];
 extern char                  s_iter[];
 extern char                  s_julibrot3d[];
@@ -564,6 +619,7 @@ extern char                  s_logmap[];
 extern char                  s_logmode[];
 extern char                  s_log[];
 extern char                  s_longitude[];
+extern char                  s_low[];
 extern char                  s_makedoc[];
 extern char                  s_makemig[];
 extern char                  s_makepar[];
@@ -573,20 +629,23 @@ extern char                  s_map[];
 extern char                  s_maxcolorres[];
 extern char                  s_maxiter[];
 extern char                  s_mcga[];
+extern char                  s_mid[];
 extern char                  s_miim[];
 extern char                  s_mod[];
 extern char                  s_mono[];
 extern char                  s_mult[];
-extern char                  s_noiit[];
 extern char                  s_none[];
 extern char                  s_noninterlaced[];
 extern char                  s_normal[];
 extern char                  s_no[];
 extern char                  s_numframes[];
+extern char                  s_off[];
 extern char                  s_olddemmcolors[];
+extern char                  s_one[];
 extern char                  s_orbitdelay[];
 extern char                  s_orbitname[];
 extern char                  s_orbitsave[];
+extern char                  s_orgfrmdir[];
 extern char                  s_origin[];
 extern char                  s_or[];
 extern char                  s_outside[];
@@ -603,11 +662,13 @@ extern char                  s_pixel[];
 extern char                  s_pi[];
 extern char                  s_play[];
 extern char                  s_plotstyle[];
+extern char                  s_polyphony[];
 extern char                  s_potential[];
 extern char far              s_pressanykeytocontinue[];
 extern char                  s_preview[];
 extern char                  s_printer[];
 extern char                  s_printfile[];
+extern char                  s_prox[];
 extern char                  s_radius[];
 extern char                  s_ramvideo[];
 extern char                  s_randomize[];
@@ -625,6 +686,7 @@ extern char                  s_round[];
 extern char                  s_rseed[];
 extern char                  s_savename[];
 extern char                  s_savetime[];
+extern char                  s_scalemap[];
 extern char                  s_scalexyz[];
 extern char                  s_showbox[];
 extern char                  s_showdot[];
@@ -637,9 +699,11 @@ extern char                  s_sphere[];
 extern char                  s_sqr[];
 extern char                  s_sqrt[];
 extern char                  s_srand[];
+extern char                  s_srelease[];
 extern char                  s_startrail[];
 extern char                  s_stereo[];
 extern char                  s_sum[];
+extern char                  s_sustain[];
 extern char                  s_symmetry[];
 extern char                  s_tanh[];
 extern char                  s_tan[];
@@ -659,8 +723,10 @@ extern char                  s_vesadetect[];
 extern char                  s_vga[];
 extern char                  s_video[];
 extern char                  s_viewwindows[];
+extern char                  s_volume[];
 extern char                  s_warn[];
 extern char                  s_waterline[];
+extern char                  s_wavetype[];
 extern char                  s_workdir[];
 extern char                  s_xaxis[];
 extern char                  s_xyadjust[];
@@ -674,6 +740,7 @@ extern char                  s_zero[];
 extern char                  s_zmag[];
 extern char                  s_z[];
 extern int                   tabmode;
+extern int                   taborhelp;
 extern int                   Targa_Out;
 extern int                   Targa_Overlay;
 extern char                  temp1[];
@@ -688,6 +755,7 @@ extern int                   textrow;
 extern int                   textsafe2;
 extern int                   textsafe;
 extern int                   text_type;
+extern unsigned int          this_gen_rseed;
 extern unsigned far *        tga16;
 extern long far *            tga32;
 extern char                  three_pass;
@@ -710,10 +778,12 @@ extern double                twopi;
 extern VOIDFARPTR            typespecific_workarea;
 extern char                  useinitorbit;
 extern BYTE                  used_extra;
+extern int                   use_grid;	
 extern BYTE                  usemag;
-extern int                   uses_p1;
-extern int                   uses_p2;
-extern int                   uses_p3;
+extern short                 uses_ismand;
+extern short                 uses_p1;
+extern short                 uses_p2;
+extern short                 uses_p3;
 extern int                   use_old_distest;
 extern int                   use_old_period;
 extern int                   using_jiim;
@@ -785,5 +855,7 @@ extern int                   zoomoff;
 extern int                   zrotate;
 extern double                zskew;
 extern double                zwidth;
-
+extern unsigned long         dif_counter;
+extern unsigned long         dif_limit;
+extern unsigned              bits;
 #endif
