@@ -246,7 +246,7 @@ void bfcornerstofloat(void)
       yy3rd = (double)bftofloat(bfy3rd);
    }
    for(i=0;i<MAXPARAMS;i++)
-      if(typehasparm(fractype,i))
+      if(typehasparm(fractype,i,NULL))
          param[i] = (double)bftofloat(bfparms[i]);
 }
 
