@@ -52,7 +52,7 @@ an appropriate setup, per_image, per_pixel, and orbit routines.
 
 
 #define NEWTONDEGREELIMIT  100
-
+#if 0 /* why are these macros here? They are also in mpmath.h */
 #define CMPLXsqr_old(out)       \
    (out).y = (old.x+old.x) * old.y;\
    (out).x = tempsqrx - tempsqry
@@ -82,7 +82,7 @@ an appropriate setup, per_image, per_pixel, and orbit routines.
      if(denom==0.0) {(out).x = 1.0e10;(out).y = 1.0e10;}else\
     { (out).x =  (arg).x/denom;\
      (out).y = -(arg).y/denom;}}
-
+#endif
 _LCMPLX lcoefficient,lold,lnew,lparm, linit,ltmp,ltmp2,lparm2;
 long ltempsqrx,ltempsqry;
 int maxcolor;
@@ -244,15 +244,6 @@ int near fpMANRbailout()
    return(0);
 }
 
-/* longbailout() is equivalent to next */
-#define LONGBAILOUT()   \
-   ltempsqrx = lsqr(lnew.x); ltempsqry = lsqr(lnew.y);\
-   lmagnitud = ltempsqrx + ltempsqry;\
-   if (lmagnitud >= llimit || lmagnitud < 0 || labs(lnew.x) > llimit2\
-         || labs(lnew.y) > llimit2) \
-               { return(1);}\
-   lold = lnew;
-
 #define FLOATTRIGBAILOUT()  \
    if (fabs(old.y) >= rqlim2) return(1);
 
@@ -951,10 +942,9 @@ UnityFractal()
 #endif
 }
 
-#define XXOne new.x
-
 UnityfpFractal()
 {
+double XXOne;
    /* brought to you by Mark Peterson - you won't find this in any fractal
       books unless they saw it here first - Mark invented it! */
 
@@ -967,8 +957,6 @@ UnityfpFractal()
    return(0);
 }
 
-#undef XXOne
-
 Mandel4Fractal()
 {
    /* By writing this code, Bert has left behind the excuse "don't
@@ -1167,7 +1155,7 @@ Richard8Fractal()
 #endif
 }
 
-PopcornFractal()
+PopcornFractal_Old()
 {
    tmp = old;
    tmp.x *= 3.0;
@@ -1203,7 +1191,46 @@ PopcornFractal()
    return(0);
 }
 
-LPopcornFractal()
+PopcornFractal()
+{
+   tmp = old;
+   tmp.x *= 3.0;
+   tmp.y *= 3.0;
+   FPUsincos(&tmp.x,&sinx,&cosx);
+   FPUsincos(&tmp.y,&siny,&cosy);
+   tmp.x = sinx/cosx + old.x;
+   tmp.y = siny/cosy + old.y;
+   FPUsincos(&tmp.x,&sinx,&cosx);
+   FPUsincos(&tmp.y,&siny,&cosy);
+   new.x = old.x - parm.x*siny;
+   new.y = old.y - parm.x*sinx;
+   /*
+   new.x = old.x - parm.x*sin(old.y+tan(3*old.y));
+   new.y = old.y - parm.x*sin(old.x+tan(3*old.x));
+   */
+   if(plot == noplot)
+   {
+      plot_orbit(new.x,new.y,1+row%colors);
+      old = new;
+   }
+   /* else */
+   /* FLOATBAILOUT(); */
+   /* PB The above line was weird, not what it seems to be!  But, bracketing
+         it or always doing it (either of which seem more likely to be what
+         was intended) changes the image for the worse, so I'm not touching it.
+         Same applies to int form in next routine. */
+   /* PB later: recoded inline, still leaving it weird */
+   /* JCO: sqr's should always be done, else magnitude could be wrong */
+   tempsqrx = sqr(new.x);
+   tempsqry = sqr(new.y);
+   if((magnitude = tempsqrx + tempsqry) >= rqlim
+     || fabs(new.x) > rqlim2 || fabs(new.y) > rqlim2 )
+           return(1);
+   old = new;
+   return(0);
+}
+
+LPopcornFractal_Old()
 {
 #ifndef XFRACT
    ltmp = lold;
@@ -1227,8 +1254,146 @@ LPopcornFractal()
       lold = lnew;
    }
    else
-      LONGBAILOUT();
+   /* LONGBAILOUT(); */
    /* PB above still the old way, is weird, see notes in FP popcorn case */
+   {
+      ltempsqrx = lsqr(lnew.x);
+      ltempsqry = lsqr(lnew.y);
+   }
+   lmagnitud = ltempsqrx + ltempsqry;
+   if (lmagnitud >= llimit || lmagnitud < 0 || labs(lnew.x) > llimit2
+         || labs(lnew.y) > llimit2)
+               return(1);
+   lold = lnew;
+   return(0);
+#endif
+}
+
+LPopcornFractal()
+{
+#ifndef XFRACT
+   ltmp = lold;
+   ltmp.x *= 3L;
+   ltmp.y *= 3L;
+   LTRIGARG(ltmp.x);
+   LTRIGARG(ltmp.y);
+   SinCos086(ltmp.x,&lsinx,&lcosx);
+   SinCos086(ltmp.y,&lsiny,&lcosy);
+   ltmp.x = divide(lsinx,lcosx,bitshift) + lold.x;
+   ltmp.y = divide(lsiny,lcosy,bitshift) + lold.y;
+   LTRIGARG(ltmp.x);
+   LTRIGARG(ltmp.y);
+   SinCos086(ltmp.x,&lsinx,&lcosx);
+   SinCos086(ltmp.y,&lsiny,&lcosy);
+   lnew.x = lold.x - multiply(lparm.x,lsiny,bitshift);
+   lnew.y = lold.y - multiply(lparm.x,lsinx,bitshift);
+   if(plot == noplot)
+   {
+      iplot_orbit(lnew.x,lnew.y,1+row%colors);
+      lold = lnew;
+   }
+   /* else */
+   /* JCO: sqr's should always be done, else magnitude could be wrong */
+   ltempsqrx = lsqr(lnew.x);
+   ltempsqry = lsqr(lnew.y);
+   lmagnitud = ltempsqrx + ltempsqry;
+   if (lmagnitud >= llimit || lmagnitud < 0
+      || labs(lnew.x) > llimit2
+         || labs(lnew.y) > llimit2)
+               return(1);
+   lold = lnew;
+   return(0);
+#endif
+}
+
+/* Popcorn generalization proposed by HB  */
+
+PopcornFractalFn()
+{
+   _CMPLX tmpx;
+   _CMPLX tmpy;
+
+   /* tmpx contains the generalized value of the old real "x" equation */
+   CMPLXtimesreal(parm2,old.y,tmp);  /* tmp = (C * old.y)         */
+   CMPLXtrig1(tmp,tmpx);             /* tmpx = trig1(tmp)         */
+   tmpx.x += old.y;                  /* tmpx = old.y + trig1(tmp) */
+   CMPLXtrig0(tmpx,tmp);             /* tmp = trig0(tmpx)         */
+   CMPLXmult(tmp,parm,tmpx);         /* tmpx = tmp * h            */
+
+   /* tmpy contains the generalized value of the old real "y" equation */
+   CMPLXtimesreal(parm2,old.x,tmp);  /* tmp = (C * old.x)         */
+   CMPLXtrig3(tmp,tmpy);             /* tmpy = trig3(tmp)         */
+   tmpy.x += old.x;                  /* tmpy = old.x + trig1(tmp) */
+   CMPLXtrig2(tmpy,tmp);             /* tmp = trig2(tmpy)         */
+
+   CMPLXmult(tmp,parm,tmpy);         /* tmpy = tmp * h            */
+
+   new.x = old.x - tmpx.x - tmpy.y;  
+   new.y = old.y - tmpy.x - tmpx.y;  
+
+   if(plot == noplot)
+   {
+      plot_orbit(new.x,new.y,1+row%colors);
+      old = new;
+   }
+
+   tempsqrx = sqr(new.x);
+   tempsqry = sqr(new.y);
+   if((magnitude = tempsqrx + tempsqry) >= rqlim
+     || fabs(new.x) > rqlim2 || fabs(new.y) > rqlim2 )
+           return(1);
+   old = new;
+   return(0);
+}
+
+#define FIX_OVERFLOW(arg) if(overflow)  \
+   { \
+      (arg).x = fudge;\
+      (arg).y = 0;\
+      overflow = 0;\
+   }
+
+LPopcornFractalFn()
+{
+#ifndef XFRACT
+   _LCMPLX ltmpx, ltmpy;
+
+   overflow = 0;
+
+   /* ltmpx contains the generalized value of the old real "x" equation */
+   LCMPLXtimesreal(lparm2,lold.y,ltmp); /* tmp = (C * old.y)         */
+   LCMPLXtrig1(ltmp,ltmpx);             /* tmpx = trig1(tmp)         */
+   FIX_OVERFLOW(ltmpx);
+   ltmpx.x += lold.y;                   /* tmpx = old.y + trig1(tmp) */
+   LCMPLXtrig0(ltmpx,ltmp);             /* tmp = trig0(tmpx)         */
+   FIX_OVERFLOW(ltmp);
+   LCMPLXmult(ltmp,lparm,ltmpx);        /* tmpx = tmp * h            */
+
+   /* ltmpy contains the generalized value of the old real "y" equation */
+   LCMPLXtimesreal(lparm2,lold.x,ltmp); /* tmp = (C * old.x)         */
+   LCMPLXtrig3(ltmp,ltmpy);             /* tmpy = trig3(tmp)         */
+   FIX_OVERFLOW(ltmpy);
+   ltmpy.x += lold.x;                   /* tmpy = old.x + trig1(tmp) */
+   LCMPLXtrig2(ltmpy,ltmp);             /* tmp = trig2(tmpy)         */
+   FIX_OVERFLOW(ltmp);
+   LCMPLXmult(ltmp,lparm,ltmpy);        /* tmpy = tmp * h            */
+
+   lnew.x = lold.x - ltmpx.x - ltmpy.y;  
+   lnew.y = lold.y - ltmpy.x - ltmpx.y;  
+
+   if(plot == noplot)
+   {
+      iplot_orbit(lnew.x,lnew.y,1+row%colors);
+      lold = lnew;
+   }
+   ltempsqrx = lsqr(lnew.x);
+   ltempsqry = lsqr(lnew.y);
+   lmagnitud = ltempsqrx + ltempsqry;
+   if (lmagnitud >= llimit || lmagnitud < 0
+      || labs(lnew.x) > llimit2
+      || labs(lnew.y) > llimit2)
+      return(1);
+   lold = lnew;
    return(0);
 #endif
 }
@@ -2185,8 +2350,8 @@ CirclelongFractal()
 /* transform points with reciprocal function */
 void invertz2(_CMPLX *z)
 {
-   z->x = dx0[col]+dx1[row];
-   z->y = dy0[row]+dy1[col];
+   z->x = dxpixel();
+   z->y = dypixel();
    z->x -= f_xcenter; z->y -= f_ycenter;  /* Normalize values to center of circle */
 
    tempsqrx = sqr(z->x) + sqr(z->y);  /* Get old radius */
@@ -2225,8 +2390,8 @@ int long_julia_per_pixel()
    }
    else
    {
-      lold.x = lx0[col]+lx1[row];
-      lold.y = ly0[row]+ly1[col];
+      lold.x = lxpixel();
+      lold.y = lypixel();
    }
    return(0);
 #else
@@ -2251,7 +2416,7 @@ int long_mandel_per_pixel()
    /* integer mandel types */
    /* barnsleym1 */
    /* barnsleym2 */
-   linit.x = lx0[col]+lx1[row];
+   linit.x = lxpixel();
 
    if(invert)
    {
@@ -2313,8 +2478,8 @@ int julia_per_pixel()
    }
    else
    {
-      lold.x = lx0[col]+lx1[row];
-      lold.y = ly0[row]+ly1[col];
+      lold.x = lxpixel();
+      lold.y = lypixel();
    }
 
    ltempsqrx = multiply(lold.x, lold.x, bitshift);
@@ -2361,7 +2526,7 @@ int mandel_per_pixel()
       linit.y = (long)(init.y*fudge);
    }
    else
-      linit.x = lx0[col]+lx1[row];
+      linit.x = lxpixel();
    switch (fractype)
      {
         case MANDELLAMBDA:              /* Critical Value 0.5 + 0.0i  */
@@ -2418,7 +2583,7 @@ int marksmandel_per_pixel()
       linit.y = (long)(init.y*fudge);
    }
    else
-      linit.x = lx0[col]+lx1[row];
+      linit.x = lxpixel();
 
    if(useinitorbit == 1)
       lold = linitorbit;
@@ -2455,7 +2620,7 @@ int marksmandelfp_per_pixel()
    if(invert)
       invertz2(&init);
    else
-      init.x = dx0[col]+dx1[row];
+      init.x = dxpixel();
 
    if(useinitorbit == 1)
       old = initorbit;
@@ -2501,7 +2666,7 @@ int mandelfp_per_pixel()
    if(invert)
       invertz2(&init);
    else
-      init.x = dx0[col]+dx1[row];
+      init.x = dxpixel();
     switch (fractype)
       {
         case MAGNET2M:
@@ -2551,8 +2716,8 @@ int juliafp_per_pixel()
       invertz2(&old);
    else
    {
-     old.x = dx0[col]+dx1[row];
-     old.y = dy0[row]+dy1[col];
+     old.x = dxpixel();
+     old.y = dypixel();
    }
    tempsqrx = sqr(old.x);  /* precalculated value for regular Julia */
    tempsqry = sqr(old.y);
@@ -2572,8 +2737,8 @@ int MPCjulia_per_pixel()
       invertz2(&old);
    else
    {
-     old.x = dx0[col]+dx1[row];
-     old.y = dy0[row]+dy1[col];
+     old.x = dxpixel();
+     old.y = dypixel();
    }
    mpcold.x = *pd2MP(old.x);
    mpcold.y = *pd2MP(old.y);
@@ -2597,7 +2762,7 @@ int othermandelfp_per_pixel()
    if(invert)
       invertz2(&init);
    else
-      init.x = dx0[col]+dx1[row];
+      init.x = dxpixel();
 
    if(useinitorbit == 1)
       old = initorbit;
@@ -2621,7 +2786,7 @@ int MPCHalley_per_pixel()
    if(invert)
       invertz2(&init);
    else
-     init.x = dx0[col]+dx1[row];
+     init.x = dxpixel();
 
    mpcold.x = *pd2MP(init.x);
    mpcold.y = *pd2MP(init.y);
@@ -2638,7 +2803,7 @@ int Halley_per_pixel()
    if(invert)
       invertz2(&init);
    else
-      init.x = dx0[col]+dx1[row];
+      init.x = dxpixel();
 
    old = init;
 
@@ -2651,8 +2816,8 @@ int otherjuliafp_per_pixel()
       invertz2(&old);
    else
    {
-      old.x = dx0[col]+dx1[row];
-      old.y = dy0[row]+dy1[col];
+      old.x = dxpixel();
+      old.y = dypixel();
    }
    return(0);
 }
@@ -2667,8 +2832,8 @@ int otherjuliafp_per_pixel()
 
 int quaternionjulfp_per_pixel()
 {
-   old.x = dx0[col]+dx1[row];
-   old.y = dy0[row]+dy1[col];
+   old.x = dxpixel();
+   old.y = dypixel();
    floatparm->x = param[4];
    floatparm->y = param[5];
    qc  = param[0];
@@ -2684,8 +2849,8 @@ int quaternionfp_per_pixel()
    old.y = 0;
    floatparm->x = 0;
    floatparm->y = 0;
-   qc  = dx0[col]+dx1[row];
-   qci = dy0[row]+dy1[col];
+   qc  = dxpixel();
+   qci = dypixel();
    qcj = param[2];
    qck = param[3];
    return(0);
@@ -2696,7 +2861,7 @@ int MarksCplxMandperp(void)
    if(invert)
       invertz2(&init);
    else
-      init.x = dx0[col]+dx1[row];
+      init.x = dxpixel();
    old.x = init.x + parm.x; /* initial pertubation of parameters set */
    old.y = init.y + parm.y;
    tempsqrx = sqr(old.x);  /* precalculated value */
@@ -2726,8 +2891,8 @@ int long_phoenix_per_pixel()
    }
    else
    {
-      lold.x = lx0[col]+lx1[row];
-      lold.y = ly0[row]+ly1[col];
+      lold.x = lxpixel();
+      lold.y = lypixel();
    }
    ltempsqrx = multiply(lold.x, lold.x, bitshift);
    ltempsqry = multiply(lold.y, lold.y, bitshift);
@@ -2746,8 +2911,8 @@ int phoenix_per_pixel()
       invertz2(&old);
    else
    {
-      old.x = dx0[col]+dx1[row];
-      old.y = dy0[row]+dy1[col];
+      old.x = dxpixel();
+      old.y = dypixel();
    }
    tempsqrx = sqr(old.x);  /* precalculated value */
    tempsqry = sqr(old.y);
@@ -2758,7 +2923,7 @@ int phoenix_per_pixel()
 int long_mandphoenix_per_pixel()
 {
 #ifndef XFRACT
-   linit.x = lx0[col]+lx1[row];
+   linit.x = lxpixel();
 
    if(invert)
    {
@@ -2799,7 +2964,7 @@ int mandphoenix_per_pixel()
    if(invert)
       invertz2(&init);
    else
-      init.x = dx0[col]+dx1[row];
+      init.x = dxpixel();
 
    if(useinitorbit == 1)
       old = initorbit;
@@ -2912,91 +3077,93 @@ EscherfpFractal() /* Science of Fractal Images pp. 185, 187 */
    }
 }
 
-#if 0
-demowalk()
+/*
+ * The following functions calculate the real and imaginary complex
+ * coordinates of the point in the complex plane corresponding to
+ * the screen coordinates (col,row) at the current zoom corners 
+ * settings. The functions come in two flavors. One looks up the pixel
+ * values using the precalculated grid arrays dx0, dx1, dy0, and dy1,
+ * which has a speed advantage but is limited to MAXPIXELS image 
+ * dimensions. The other calculates the complex coordinates at a
+ * cost of two additions and two multiplications for each component,
+ * but works at any resolution.
+ *
+ * With Microsoft C's _fastcall keyword, the function call overhead
+ * appears to be negligible. It also appears that the speed advantage
+ * of the lookup vs the calculation is negligible on machines with
+ * coprocessors. Bert Tyler's original implementation was designed for
+ * machines with no coprocessor; on those machines the saving was
+ * significant. For the time being, the table lookup capability will
+ * be maintained.
+ */
+
+/* Real component, grid lookup version - requires dx0/dx1 arrays */
+static double _fastcall dxpixel_grid(void)
 {
-    float stepsize;                     /* average stepsize */
-    int xwalk, ywalk;                   /* current position */
-    int xstep, ystep;                   /* current step */
-    long steps;                         /* number of steps */
-    int color;                          /* color to draw this step */
-    float temp, tempadjust;             /* temporary variables */
+   return(dx0[col]+dx1[row]);
+}
 
-if (param[0] != 999) {                  /* if 999, do a Mandelbrot instead */
+/* Real component, calculation version - does not require arrays */
+static double _fastcall dxpixel_calc(void)
+{
+   return((double)(xxmin + col*delxx + row*delxx2));
+}
 
-    srand(rseed);                       /* seed the random number generator */
-    if (!rflag) ++rseed;
-    tempadjust = RAND_MAX >> 2;         /* adjustment factor */
+/* Imaginary component, grid lookup version - requires dy0/dy1 arrays */
+static double _fastcall dypixel_grid(void)
+{
+   return(dy0[row]+dy1[col]);
+}
 
-    xwalk = xdots / 2;                  /* start in the center of the image */
-    ywalk = ydots / 2;
+/* Imaginary component, calculation version - does not require arrays */
+static double _fastcall dypixel_calc(void)
+{
+   return((double)(yymax - row*delyy - col*delyy2));
+}
 
-    stepsize = min(xdots, ydots)        /* calculate average stepsize */
-               * (param[0]/100.0);      /* as a percentage of the image */
+/* Real component, grid lookup version - requires lx0/lx1 arrays */
+static long _fastcall lxpixel_grid(void)
+{
+   return(lx0[col]+lx1[row]);
+}
 
-    color = (int)max(0, min(colors, param[1]));  /* set the initial color */
+/* Real component, calculation version - does not require arrays */
+static long _fastcall lxpixel_calc(void)
+{
+   return(xmin + col*delx + row*delx2);
+}
 
-    for (steps = 0; steps < maxit; steps++) { /* take maxit steps */
-        if (keypressed())               /* abort if told to do so */
-            return(0);
-        temp = rand();                  /* calculate the next xstep */
-        xstep = (int)(((temp/tempadjust) - 2.0) * stepsize);
-        xstep = min(xwalk + xstep, xdots - 1);
-        xstep = max(0, xstep);
-        temp = rand();                  /* calculate the next ystep */
-        ystep = (int)(((temp/tempadjust) - 2.0) * stepsize);
-        ystep = min(ywalk + ystep, ydots - 1);
-        ystep = max(0, ystep);
-        if (param[1] == 0.0)            /* rotate the colors? */
-            if (++color >= colors)      /* rotate the colors, avoiding */
-                color = 1;              /* the background color 0 */
-        /* the draw_line function is borrowed from the 3D routines */
-        draw_line(xwalk, ywalk,xstep,ystep,color);
-        /* or, we could be on a pogo stick and just displaying
-           where we landed...
-        putcolor(xstep, ystep, color);
-        */
+/* Imaginary component, grid lookup version - requires ly0/ly1 arrays */
+static long _fastcall lypixel_grid(void)
+{
+   return(ly0[row]+ly1[col]);
+}
 
-        xwalk = xstep;                  /* remember where we were */
-        ywalk = ystep;
-        }
-    return(1);                          /* we're done */
+/* Imaginary component, calculation version - does not require arrays */
+static long _fastcall lypixel_calc(void)
+{
+   return(ymax - row*dely - col*dely2);
+}
 
-} else {                                /* a simple Mandelbrot routine */
+double (_fastcall *dxpixel)(void) = dxpixel_calc;
+double (_fastcall *dypixel)(void) = dypixel_calc;
+long   (_fastcall *lxpixel)(void) = lxpixel_calc;
+long   (_fastcall *lypixel)(void) = lypixel_calc;
 
-    /* the following routine determines the X and Y values of
-       each pixel coordinate and calculates a simple mandelbrot
-       fractal with them - slowly, but surely */
-    int ix, iy;
-    for (iy = 0; iy < ydots; iy++) {
-        for (ix = 0; ix < xdots; ix++) {
-            long iter;
-            double x, y, newx, newy, tempxx, tempxy, tempyy;
-            /* first, obtain the X and Y coordinate values of this pixel */
-            x = dx0[ix]+dx1[iy];
-            y = dy0[iy]+dy1[ix];
-            /* now initialize the temporary values */
-            tempxx = tempyy = tempxy = 0.0;
-            if (keypressed())           /* abort if told to do so */
-                return(0);
-            /* the inner iteration loop */
-            for (iter = 1; iter < maxit; iter++) {
-                /* calculate the X and Y values of Z(iter) */
-                newx = tempxx - tempyy + x;
-                newy = tempxy + tempxy + y;
-                /* calculate the temporary values */
-                tempxx = newx * newx;
-                tempyy = newy * newy;
-                tempxy = newx * newy;
-                /* are we done yet? */
-                if (tempxx + tempyy > 4.0) break;
-                }
-            /* color in the pixel */
-            putcolor(ix, iy, (int)iter % colors);
-            }
-        }
-    return(1);                          /* we're done */
-    }
-
+void set_pixel_calc_functions(void)
+{
+   if(use_grid)
+   {
+      dxpixel = dxpixel_grid;
+      dypixel = dypixel_grid;
+      lxpixel = lxpixel_grid;
+      lypixel = lypixel_grid;
+   }   
+   else
+   {
+      dxpixel = dxpixel_calc;
+      dypixel = dypixel_calc;
+      lxpixel = lxpixel_calc;
+      lypixel = lypixel_calc;
+   }
 }
-#endif
