@@ -21,9 +21,11 @@
 
 MandelSetup()           /* Mandelbrot Routine */
 {
-   if (debugflag != 90 && ! invert && decomp[0] == 0 && rqlim == 4.0
+   if (debugflag != 90
+       && !invert && decomp[0] == 0 && rqlim == 4.0
        && bitshift == 29 && potflag == 0
        && biomorph == -1 && inside > -59 && outside >= -1
+       && inside != FMOD
        && useinitorbit != 1 && using_jiim == 0 && bailoutest == Mod
        && (orbitsave&2) == 0)
       calctype = calcmand; /* the normal case - use CALCMAND */
@@ -38,9 +40,11 @@ MandelSetup()           /* Mandelbrot Routine */
 
 JuliaSetup()            /* Julia Routine */
 {
-   if (debugflag != 90 && ! invert && decomp[0] == 0 && rqlim == 4.0
+   if (debugflag != 90 
+       && !invert && decomp[0] == 0 && rqlim == 4.0
        && bitshift == 29 && potflag == 0
        && biomorph == -1 && inside > -59 && outside >= -1
+       && inside != FMOD
        && !finattract && using_jiim == 0 && bailoutest == Mod
        && (orbitsave&2) == 0)
        calctype = calcmand; /* the normal case - use CALCMAND */
@@ -212,20 +216,43 @@ MandelfpSetup()
            calcmandfp() can currently handle invert, any rqlim, potflag
            zmag, epsilon cross, and all the current outside options
                                                      Wes Loewer 11/03/91
+           Took out support for inside= options, for speed. 7/13/97
         */
-        if (debugflag != 90
+        if (debugflag != 90 
             && !distest
             && decomp[0] == 0
             && biomorph == -1
-            && (inside >= -1 || inside == -59 || inside == -100)
+            && (inside >= -1)
             /* uncomment this next line if more outside options are added */
             /* && outside >= -5 */
             && useinitorbit != 1
+            && (soundflag & 0x07) < 2
             && using_jiim == 0 && bailoutest == Mod
             && (orbitsave&2) == 0)
         {
            calctype = calcmandfp; /* the normal case - use calcmandfp */
-           calcmandfpasmstart();
+#ifndef XFRACT
+           if (cpu >= 386 && fpu >= 387)
+           {
+              calcmandfpasmstart_p5();
+              calcmandfpasm = (long (*)(void))calcmandfpasm_p5;
+           }
+           else if (cpu == 286 && fpu >= 287)
+           {
+              calcmandfpasmstart();
+              calcmandfpasm = (long (*)(void))calcmandfpasm_287;
+           }
+           else
+
+           {
+              calcmandfpasmstart();
+              calcmandfpasm = (long (*)(void))calcmandfpasm_87;
+           }
+#else
+           {
+              calcmandfpasmstart();
+           }
+#endif
         }
         else
         {
@@ -313,21 +340,43 @@ JuliafpSetup()
            calcmandfp() can currently handle invert, any rqlim, potflag
            zmag, epsilon cross, and all the current outside options
                                                      Wes Loewer 11/03/91
+           Took out support for inside= options, for speed. 7/13/97
         */
-        if (debugflag != 90
+        if (debugflag != 90 
             && !distest
             && decomp[0] == 0
             && biomorph == -1
-            && (inside >= -1 || inside == -59 || inside == -100)
+            && (inside >= -1)
             /* uncomment this next line if more outside options are added */
             /* && outside >= -5 */
             && useinitorbit != 1
+            && (soundflag & 0x07) < 2
             && !finattract
             && using_jiim == 0 && bailoutest == Mod
             && (orbitsave&2) == 0)
         {
            calctype = calcmandfp; /* the normal case - use calcmandfp */
-           calcmandfpasmstart();
+#ifndef XFRACT
+           if (cpu >= 386 && fpu >= 387)
+           {
+              calcmandfpasmstart_p5();
+              calcmandfpasm = (long (*)(void))calcmandfpasm_p5;
+           }
+           else if (cpu == 286 && fpu >= 287)
+           {
+              calcmandfpasmstart();
+              calcmandfpasm = (long (*)(void))calcmandfpasm_287;
+           }
+           else
+           {
+              calcmandfpasmstart();
+              calcmandfpasm = (long (*)(void))calcmandfpasm_87;
+           }
+#else
+           {
+              calcmandfpasmstart();
+           }
+#endif
         }
         else
         {
@@ -384,6 +433,31 @@ JuliafpSetup()
       if(param[4] != 0.0 || param[5] != 0)
          symmetry = NOSYM;
       break;
+   case FPPOPCORN:
+   case FPPOPCORNJUL:
+      {
+         int default_functions = 0;
+         if(trigndx[0] == SIN &&
+            trigndx[1] == TAN &&
+            trigndx[2] == SIN &&
+            trigndx[3] == TAN &&
+            fabs(parm2.x - 3.0) < .0001 &&
+            parm2.y == 0 &&
+            parm.y == 0)
+         {
+            default_functions = 1;
+            if(fractype == FPPOPCORNJUL)
+               symmetry = ORIGIN;
+         }   
+         if(save_release <=1960)
+            curfractalspecific->orbitcalc = PopcornFractal_Old;
+         else if(default_functions && debugflag == 96)
+            curfractalspecific->orbitcalc = PopcornFractal; 
+         else
+            curfractalspecific->orbitcalc = PopcornFractalFn;
+         get_julia_attractor (0.0, 0.0);   /* another attractor? */
+      }
+      break;
    default:
       get_julia_attractor (0.0, 0.0);   /* another attractor? */
       break;
@@ -474,6 +548,31 @@ JulialongSetup()
         symmetry = NOSYM;
       get_julia_attractor (0.0, 0.0);   /* another attractor? */
       break;
+   case LPOPCORN:
+   case LPOPCORNJUL:
+      {   
+         int default_functions = 0;
+         if(trigndx[0] == SIN &&
+            trigndx[1] == TAN &&
+            trigndx[2] == SIN &&
+            trigndx[3] == TAN &&
+            fabs(parm2.x - 3.0) < .0001 &&
+            parm2.y == 0 &&
+            parm.y == 0)
+         {
+            default_functions = 1;
+            if(fractype == LPOPCORNJUL)
+               symmetry = ORIGIN;
+         }   
+         if(save_release <=1960)
+            curfractalspecific->orbitcalc = LPopcornFractal_Old;
+         else if(default_functions && debugflag == 96)
+            curfractalspecific->orbitcalc = LPopcornFractal;
+         else
+            curfractalspecific->orbitcalc = LPopcornFractalFn;
+         get_julia_attractor (0.0, 0.0);   /* another attractor? */
+      }
+      break;
    default:
       get_julia_attractor (0.0, 0.0);   /* another attractor? */
       break;
