@@ -30,7 +30,6 @@ BYTE olddacbox[256][3];
 extern int tabmode;
 
 int DivideOverflow = 0;
-int iit=0;
 int cpu=0;		/* cpu type: 86, 186, 286, or 386 */
 #ifndef XFRACT
 int fpu=0;		/* fpu type: 0, 87, 287, 387 */
@@ -553,33 +552,6 @@ xmmmoveextended(a)
 struct XMM_Move *a;
 {}
 
-/*
- * IIT coprocessor routines.
- */
-
-void
-load_mat(matrix)
-double (*matrix)[4];
-{}
-
-void
-mult_vec_iit(vector)
-double vector[3];
-{}
-
-int
-IITCoPro() {
-    return 0;
-}
-
-int F4x4Check()
-{
-}
-
-void F4x4Free() {}
-
-int F4x4Lock() {}
-
 
 /* --------------------------------------------------------------------
  * The following routines are used for encoding/decoding gif images.
@@ -732,6 +704,8 @@ decode_fractal_info(info,dir)
     getDouble(&info->dinvert[2],&bufPtr,dir);
     getInt(&info->logcalc,&bufPtr,dir);
     getInt(&info->stoppass,&bufPtr,dir);
+    getInt(&info->quick_calc,&bufPtr,dir);
+    getDouble(&info->closeprox,&bufPtr,dir);
 
     for (i=0;i<(sizeof(info->future)/sizeof(short));i++) {
         getInt(&info->future[i],&bufPtr,dir);
@@ -800,11 +774,11 @@ static getLong(dst,src,dir)
     } else {
 	(*src)[0] = (*dst)&0xff;
 	(*src)[1] = ((*dst)&0xff00)>>8;
-	(*src)[2] = ((*dst)&0xff0000)>>8;
+	(*src)[2] = ((*dst)&0xff0000)>>16;
 #ifdef __SVR4
-	(*src)[3] = (unsigned)((*dst)&0xff000000)>>8;
+	(*src)[3] = (unsigned)((*dst)&0xff000000)>>24;
 #else
-	(*src)[3] = ((*dst)&0xff000000)>>8;
+	(*src)[3] = ((*dst)&0xff000000)>>24;
 #endif
     }
     (*src) += 4; /* sizeof(long) in MS_DOS */
@@ -988,3 +962,58 @@ fix_ranges(ranges,num,dir)
     }
     free((char *)buf);
 }
+
+decode_evolver_info(info,dir)
+    struct evolution_info *info;
+    int dir;
+{
+    unsigned char *buf;
+    unsigned char *bufPtr;
+    int i;
+
+    if (dir==1) {
+	buf = (unsigned char *)malloc(EVOLVER_INFO_SIZE);
+	bufPtr = buf;
+	bcopy((char *)info,(char *)buf,EVOLVER_INFO_SIZE);
+    }  else {
+	buf = (unsigned char *)malloc(sizeof(struct evolution_info));
+	bufPtr = buf;
+	bcopy((char *)info,(char *)buf,sizeof(struct evolution_info));
+    }
+
+    getInt(&info->evolving,&bufPtr,dir);
+    getInt(&info->gridsz,&bufPtr,dir);
+    getInt(&info->this_gen_rseed,&bufPtr,dir);
+    getDouble(&info->fiddlefactor,&bufPtr,dir);
+    getDouble(&info->paramrangex,&bufPtr,dir);
+    getDouble(&info->paramrangey,&bufPtr,dir);
+    getDouble(&info->opx,&bufPtr,dir);
+    getDouble(&info->opy,&bufPtr,dir);
+    getInt(&info->odpx,&bufPtr,dir);
+    getInt(&info->odpy,&bufPtr,dir);
+    getInt(&info->px,&bufPtr,dir);
+    getInt(&info->py,&bufPtr,dir);
+    getInt(&info->sxoffs,&bufPtr,dir);
+    getInt(&info->syoffs,&bufPtr,dir);
+    getInt(&info->xdots,&bufPtr,dir);
+    getInt(&info->ydots,&bufPtr,dir);
+    for (i=0;i<NUMGENES;i++) {
+        getInt(&info->mutate[i],&bufPtr,dir);
+    }
+    getInt(&info->ecount,&bufPtr,dir);
+
+    for (i=0;i<(sizeof(info->future)/sizeof(short));i++) {
+        getInt(&info->future[i],&bufPtr,dir);
+    }   
+    if (bufPtr-buf != EVOLVER_INFO_SIZE) {
+	printf("Warning: loadfile miscount on evolution_info structure.\n");
+	printf("Components add up to %d bytes, but EVOLVER_INFO_SIZE = %d\n",
+		bufPtr-buf, EVOLVER_INFO_SIZE);
+    } 
+    if (dir==0) {
+	bcopy((char *)buf,(char *)info,EVOLVER_INFO_SIZE);
+    }
+
+    free(buf);
+}
+
