@@ -123,17 +123,16 @@ static long num_tris; /* number of triangles output to ray trace file */
 struct f_point far *f_lastrow;
 void (_fastcall * standardplot) (int, int, int);
 MATRIX m; /* transformation matrix */
-void (*mult_vec) (VECTOR) = mult_vec_c;
 int Ambient;
 int RANDOMIZE;
 int haze;
 int Real_V = 0; /* mrr Actual value of V for fillytpe>4 monochrome images */
-char light_name[80] = "fract001";
+char light_name[FILE_MAX_PATH] = "fract001";
 int Targa_Overlay, error;
 char targa_temp[14] = "fractemp.tga";
 int P = 250; /* Perspective dist used when viewing light vector */
 BYTE back_color[3];
-char ray_name[80] = "fract001";
+char ray_name[FILE_MAX_PATH] = "fract001";
 char preview = 0;
 char showbox = 0;
 int previewfactor = 20;
@@ -208,13 +207,14 @@ int line3d(BYTE * pixels, unsigned linelen)
       int err;
       if ((err = first_time(linelen, v)) != 0)
          return (err);
+      if(xdots > OLDMAXPIXELS)
+         return(-1);
       tout = 0;
       crossavg[0] = 0;
       crossavg[1] = 0;
       crossavg[2] = 0;
       xcenter0 = (int) (xcenter = xdots / 2 + xshift);
       ycenter0 = (int) (ycenter = ydots / 2 - yshift);
-
    }
    /* make sure these pixel coordinates are out of range */
    old = bad;
@@ -2608,13 +2608,6 @@ static int first_time(int linelen, VECTOR v)
       f_lastrow[i] = f_bad;
    }
    got_status = 3;
-   if (iit > 0)
-   {
-      load_mat(m);              /* load matrix into iit registers */
-      mult_vec = mult_vec_iit;
-   }
-   else
-      mult_vec = mult_vec_c;
    return (0);
 } /* end of once-per-image intializations */
 
@@ -2686,7 +2679,7 @@ static int line3dmem(void)
          /* not using extra segment so decrement check_extra */
          check_extra -= sizeof(struct minmax) * ydots;
          if (got_mem == NULL)
-            got_mem = (struct minmax far *) (farmemalloc(MAXPIXELS *
+            got_mem = (struct minmax far *) (farmemalloc(OLDMAXPIXELS *
                                                     sizeof(struct minmax)));
          if (got_mem)
             minmax_x = got_mem;
@@ -2720,4 +2713,3 @@ static int line3dmem(void)
 #pragma optimize( "g", on )
 #endif
 #endif
-
