@@ -148,6 +148,9 @@ Press F1 for help, "};
          break;
          }
       }
+   /* exit in makepar mode if no exact match of video mode in file */
+   if(*s_makepar == '\0' && initmode == -1)
+      return(0);
 
    if (initmode == -1) /* try to find very good match for vid mode */
       for (i = 0; i < vidtbllen; ++i) {
@@ -201,8 +204,9 @@ if (fastrestore  && !askvideo)
 
 #ifndef XFRACT
    gotrealmode = 0;
-   if (initmode < 0 || (askvideo && !initbatch)) {
-      /* no exact match or (askvideo=yes and batch=no), talk to user */
+   if ((initmode < 0 || (askvideo && !initbatch)) && *s_makepar != '\0') {
+      /* no exact match or (askvideo=yes and batch=no), and not
+        in makepar mode, talk to user */
 
       qsort(vid,vidtbllen,sizeof(vid[0]),vidcompare); /* sort modes */
 
@@ -383,13 +387,13 @@ if (fastrestore  && !askvideo)
       else
          viewreduction = tmpreduce; /* ok, this works */
       }
-   if (!fastrestore && (fabs(finalaspectratio - screenaspect) > .00001 || viewxdots != 0)) {
+   if (*s_makepar && !fastrestore && !initbatch &&
+        (fabs(finalaspectratio - screenaspect) > .00001 || viewxdots != 0)) {
       static FCODE msg[] = {"\
 Warning: <V>iew parameters are being set to non-standard values.\n\
 Remember to reset them when finished with this image!"};
       stopmsg(4,msg);
       }
-
    return(0);
 }
 
