@@ -356,8 +356,6 @@ static void free_lcmds(void)
     farmemfree(*sc++);
 }
 
-
-
 #ifdef XFRACT
 #define lsysi_doslash_386 lsysi_doslash
 #define lsysi_dobslash_386 lsysi_dobslash
@@ -609,15 +607,13 @@ findsize(struct lsys_cmd far *command, struct lsys_turtlestatei *ts, struct lsys
 if (overflow)     /* integer math routines overflowed */
     return NULL;
 
-#ifndef __TURBOC__
    if (stackavail() < 400) { /* leave some margin for calling subrtns */
       ts->stackoflow = 1;
       return NULL;
-      }
-#endif
+   }
 
    while (command->ch && command->ch !=']') {
-      static FCODE thinking_msg[] = 
+      static FCODE thinking_msg[] =
          {"L-System thinking (higher orders take longer)"};
       if (! (ts->counter++)) {
          /* let user know we're not dead */
@@ -727,15 +723,14 @@ drawLSysI(struct lsys_cmd far *command,struct lsys_turtlestatei *ts, struct lsys
    struct lsys_cmd far **rulind;
    int tran;
 
-if (overflow)     /* integer math routines overflowed */
-    return NULL;
+   if (overflow)     /* integer math routines overflowed */
+      return NULL;
 
-#ifndef __TURBOC__
    if (stackavail() < 400) { /* leave some margin for calling subrtns */
       ts->stackoflow = 1;
       return NULL;
-      }
-#endif
+   }
+
 
    while (command->ch && command->ch !=']') {
       if (!(ts->counter++)) {
