@@ -21,7 +21,7 @@
 
 /* routines in this module      */
 
-static void write_batch_parms(char *colorinf,int maxcolor,int i, int j);
+static void write_batch_parms(char *colorinf,int colorsonly, int maxcolor,int i, int j);
 static void expand_comments(char far *target, char far *source);
 
 #ifndef XFRACT
@@ -32,7 +32,7 @@ static void put_parm();
 
 static void put_parm_line(void);
 static int getprec(double,double,double);
-extern int getprecbf(int);
+int getprecbf(int);
 static void put_float(int,double,int);
 static void put_bf(int slash,bf_t r, int prec);
 static void put_filename(char *keyword,char *fname);
@@ -55,6 +55,7 @@ char s_seqd[]     = " %s=%d";
 char s_seqdd[]    = " %s=%d/%d";
 char s_seqddd[]   = " %s=%d/%d/%d";
 char s_seqldddd[]  = " %s=%ld/%d/%d/%d";
+char s_seqd12[]   = " %s=%d/%d/%d/%d/%d/%d/%d/%d/%d/%d/%d/%d";
 char s_seqy[]     = " %s=y";
 char s_x[]        = "x";
 char s_y[]        = "y";
@@ -80,6 +81,7 @@ FILE *parmfile;
 void make_batch_file()
 {
 #define MAXPROMPTS 18
+   int colorsonly = 0;
    static char far hdg[]={"Save Current Parameters"};
    /** added for pieces feature **/
    double pdelx;
@@ -101,7 +103,7 @@ void make_batch_file()
    char far * choices[MAXPROMPTS];
    char far *ptr;
    int gotinfile;
-   char outname[81], buf[256], buf2[128];
+   char outname[FILE_MAX_PATH+1], buf[256], buf2[128];
    FILE *infile;
    FILE *fpbat = NULL;
    char colorspec[14];
@@ -110,6 +112,9 @@ void make_batch_file()
    char *sptr, *sptr2;
    int oldhelpmode;
 
+   if(s_makepar[1] == 0) /* makepar map case */
+      colorsonly = 1;
+      
    /* put comment storage in extraseg */
    inpcommandfile = MK_FP(extraseg,0);
    inpcommandname = inpcommandfile+80;
@@ -125,8 +130,7 @@ void make_batch_file()
    helpmode = HELPPARMFILE;
 
    maxcolor = colors;
-   colorspec[0] = 'y';
-   colorspec[1] = '\0';
+   strcpy(colorspec,"y");
    if (gotrealdac && !reallyega)
    {
       --maxcolor;
@@ -158,7 +162,7 @@ void make_batch_file()
          sptr = colorfile;
       }
       else                      /* colors match no .map that we know of */
-         colorspec[0] = 'y';
+         strcpy (colorspec,"y");
 
       if (colorspec[0] == '@')
       {
@@ -215,7 +219,7 @@ prompt_user:
          LOADBATCHPROMPTS("Record colors?");
          paramvalues[promptnum].type = 0x100 + 13;
          paramvalues[promptnum++].uval.sbuf = colorspec;
-         LOADBATCHPROMPTS("    (no | yes for full info | @filename to point to a map file)");
+         LOADBATCHPROMPTS("    (no | yes | only for full info | @filename to point to a map file)");
          paramvalues[promptnum++].type = '*';
          LOADBATCHPROMPTS("# of colors");
          maxcolorindex = promptnum;
@@ -224,6 +228,9 @@ prompt_user:
          LOADBATCHPROMPTS("    (if recording full color info)");
          paramvalues[promptnum++].type = '*';
       }
+      LOADBATCHPROMPTS("Maximum line length");
+      paramvalues[promptnum].type = 'i';
+      paramvalues[promptnum++].uval.ival = maxlinelength;
       LOADBATCHPROMPTS("");
       paramvalues[promptnum++].type = '*';
       LOADBATCHPROMPTS("    **** The following is for generating images in pieces ****");
@@ -244,6 +251,12 @@ prompt_user:
       if (fullscreen_prompt(hdg,promptnum, choices, paramvalues, 0, NULL) < 0)
          break;
 
+      if(*colorspec == 'o' || s_makepar[1] == 0)
+      {
+         strcpy(colorspec,"y");
+         colorsonly = 1;
+      }
+
       far_strcpy(CommandFile, inpcommandfile);
       if (has_ext(CommandFile) == NULL)
          strcat(CommandFile, ".par");   /* default extension .par */
@@ -254,8 +267,15 @@ prompt_user:
          if (paramvalues[maxcolorindex].uval.ival > 0 &&
              paramvalues[maxcolorindex].uval.ival <= 256)
             maxcolor = paramvalues[maxcolorindex].uval.ival;
-
       promptnum = piecespromts;
+      {
+         int newmaxlinelength;
+         newmaxlinelength = paramvalues[promptnum-3].uval.ival;
+         if(maxlinelength != newmaxlinelength &&
+              newmaxlinelength >= MINMAXLINELENGTH &&
+              newmaxlinelength <= MAXMAXLINELENGTH)
+            maxlinelength = newmaxlinelength;    
+      }
       xm = paramvalues[promptnum++].uval.ival;
 
       ym = paramvalues[promptnum++].uval.ival;
@@ -305,7 +325,10 @@ skip_UI:
             strcpy(colorspec, "y");
          else
             strcpy(colorspec, "n");
-         maxcolor = filecolors;
+         if(s_makepar[1] == 0)
+            maxcolor = 256;
+         else   
+            maxcolor = filecolors;
       }
       strcpy(outname, CommandFile);
       gotinfile = 0;
@@ -438,7 +461,7 @@ Continue to replace it, Cancel to back out"};
             for(last=-1,i=0;i<4;i++)
                if(*par_comment[i])
                   last=i;
-            for(i=0;i<last;i++)      
+            for(i=0;i<last;i++)
                if(*CommandComment[i]=='\0')
                   far_strcpy(CommandComment[i],";");
          }
@@ -454,11 +477,11 @@ Continue to replace it, Cancel to back out"};
             for(k=1;k<4;k++)
                if (CommandComment[k][0])
                   fprintf(parmfile, "%s%s\n", buf, CommandComment[k]);
-            if (patchlevel != 0)
+            if (patchlevel != 0 && colorsonly == 0)
                fprintf(parmfile, "%s Version %d Patchlevel %d\n", buf,
                   release, patchlevel); 
          }
-         write_batch_parms(colorspec, maxcolor, i, j);
+         write_batch_parms(colorspec, colorsonly, maxcolor, i, j);
          if(xm > 1 || ym > 1)
          {
             fprintf(parmfile,"  video=%s", vidmde);
@@ -509,7 +532,7 @@ static struct write_batch_data { /* buffer for parms to break lines nicely */
    char *buf;
    } *wbdata;
 
-static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
+static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int ii, int jj)
 {
    char far *saveshared;
    int i,j,k;
@@ -536,7 +559,8 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
    far_memcpy(saveshared,boxx,10000);
    far_memset(boxx,0,10000);
    wb_data.buf = (char *)boxx;
-
+   if(colorsonly)
+      goto docolors;
    if (display3d <= 0) { /* a fractal was generated */
 
       /****** fractal only parameters in this section *******/
@@ -575,6 +599,8 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
       {
          put_filename(s_formulafile,FormFileName);
          put_parm( s_seqs,s_formulaname,FormName);
+         if (uses_ismand)
+            put_parm(" %s=%c",s_ismand,ismand?'y':'n');
       }
       if (fractype == LSYSTEM)
       {
@@ -596,6 +622,7 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
       if (usr_stdcalcmode != 'g')
          put_parm(" %s=%c",s_passes,usr_stdcalcmode);
 
+
       if (stoppass != 0)
          put_parm(" %s=%c%c",s_passes,usr_stdcalcmode,(char)stoppass + '0');
 
@@ -685,7 +712,7 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
 
       for(i = (MAXPARAMS-1); i >= 0; --i)
           if(typehasparm((fractype==JULIBROT || fractype==JULIBROTFP)
-                          ?neworbittype:fractype,i) != NULL) break;
+                          ?neworbittype:fractype,i,NULL)) break;
 
       if (i >= 0) {
         if (fractype == CELLULAR || fractype == ANT)
@@ -727,7 +754,7 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
       if(bailout && (potflag == 0 || potparam[2] == 0.0))
          put_parm(" %s=%ld",s_bailout,bailout);
 
-      if(bailoutest != Mod && (potflag == 0 || potparam[2] == 0.0)) {
+      if(bailoutest != Mod) {
          put_parm(" %s=",s_bailoutest);
          if (bailoutest == Real)
             put_parm( s_real);
@@ -764,9 +791,16 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
             put_parm(s_startrail);
          else if (inside == -102)
             put_parm(s_period);
+         else if (inside == -103)
+            put_parm(s_fmod);
+         else if (inside == -104)
+            put_parm(s_atan);
          else
             put_parm( "%d",inside);
          }
+      if (closeprox != 0.01 && (inside == EPSCROSS || inside == FMOD) ) {
+         put_parm(" %s=%.15g",s_prox,closeprox);
+         }
       if (outside != -1)
       {
          put_parm(" %s=",s_outside);
@@ -794,7 +828,7 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
             put_parm( "%ld", LogFlag);
          }
 
-      if(Log_Fly_Calc) {
+      if(Log_Fly_Calc && LogFlag) {
          put_parm( " %s=",s_logmode);
          if(Log_Fly_Calc == 1)
             put_parm( "fly");
@@ -944,6 +978,91 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
          put_parm("/%s",s_no);
       put_parm("/%d/%d",viewxdots,viewydots);
    }
+
+   if(colorsonly == 0)
+   {
+   if (rotate_lo != 1 || rotate_hi != 255)
+      put_parm( s_seqdd,s_cyclerange,rotate_lo,rotate_hi);
+
+   if(basehertz != 440)
+      put_parm(s_seqd,s_hertz,basehertz);
+
+  if(soundflag != 9) {
+   if((soundflag&7) == 0)
+      put_parm(s_seqs,s_sound,s_off);
+   else if((soundflag&7) == 1)
+      put_parm(s_seqs,s_sound,s_beep);
+   else if((soundflag&7) == 2)
+      put_parm(s_seqs,s_sound,s_x);
+   else if((soundflag&7) == 3)
+      put_parm(s_seqs,s_sound,s_y);
+   else if((soundflag&7) == 4)
+      put_parm(s_seqs,s_sound,s_z);
+#ifndef XFRACT
+   if((soundflag&7) && (soundflag&7) <=4) {
+      if(soundflag&8)
+         put_parm("/pc");
+      if(soundflag&16)
+         put_parm("/fm");
+      if(soundflag&32)
+         put_parm("/midi");
+      if(soundflag&64)
+         put_parm("/quant");
+   }
+#endif
+  }
+
+#ifndef XFRACT
+   if(fm_vol != 63)
+     put_parm(s_seqd,s_volume,fm_vol);
+
+   if(hi_atten != 0) {
+     if(hi_atten == 1)
+        put_parm(s_seqs,s_atten,s_low);
+     else if(hi_atten == 2)
+        put_parm(s_seqs,s_atten,s_mid);
+     else if(hi_atten == 3)
+        put_parm(s_seqs,s_atten,s_high);
+     else   /* just in case */
+        put_parm(s_seqs,s_atten,s_none);
+   }
+
+   if(polyphony != 0)
+     put_parm(s_seqd,s_polyphony,polyphony+1);
+   
+   if(fm_wavetype !=0)
+     put_parm(s_seqd,s_wavetype,fm_wavetype); 
+   
+   if(fm_attack != 5)
+      put_parm(s_seqd,s_attack,fm_attack);
+
+   if(fm_decay != 10)
+      put_parm(s_seqd,s_decay,fm_decay);
+
+   if(fm_sustain != 13)
+      put_parm(s_seqd,s_sustain,fm_sustain);
+
+   if(fm_release != 5)
+      put_parm(s_seqd,s_srelease,fm_release);
+
+   if(soundflag&64) { /* quantize turned on */
+      for(i=0;i<=11;i++) if(scale_map[i] != i+1) i=15;
+      if(i>12) 
+         put_parm(s_seqd12,s_scalemap,scale_map[0],scale_map[1],scale_map[2],scale_map[3]
+            ,scale_map[4],scale_map[5],scale_map[6],scale_map[7],scale_map[8]
+            ,scale_map[9],scale_map[10],scale_map[11]);
+   }
+
+#endif
+
+   if(orbit_delay > 0)
+      put_parm(s_seqd,s_orbitdelay,orbit_delay);
+
+   if(start_showorbit > 0)
+      put_parm(s_seqs,s_showorbit,s_yes);
+
+   }
+
    if (*colorinf != 'n') 
    {
       if(recordcolors=='c' && *colorinf == '@')
@@ -953,6 +1072,7 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
          put_parm(colorinf);
          put_parm_line();
       }   
+docolors:
       put_parm(" %s=",s_colors);
       if (recordcolors !='c' && recordcolors != 'y' && *colorinf == '@')
          put_parm(colorinf);
@@ -972,6 +1092,8 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
             put_parm(buf);
             if (++curc >= maxcolor)      /* quit if done last color */
                break;
+            if(debugflag == 920)  /* lossless compression */
+               continue;   
             /* Next a P Branderhorst special, a tricky scan for smooth-shaded
                ranges which can be written as <nn> to compress .par file entry.
                Method used is to check net change in each color value over
@@ -991,6 +1113,12 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
                   i = 3;
                for (k = 0; k <= i; ++k) {
                   for (j = 0; j < 3; ++j) { /* check pattern of chg per color */
+                     /* Sylvie Gallet's fix */
+                     if (debugflag != 910 && scanc > (curc+4) && scanc < maxcolor-5)
+                        if (abs(2*dacbox[scanc][j] - dacbox[scanc-5][j]
+                                - dacbox[scanc+5][j]) >= 2)
+                           break;
+                     /* end Sylvie's fix */       
                      delta = (int)dacbox[scanc][j] - (int)dacbox[scanc-k-1][j];
                      if (k == scanc - curc)
                         diff1[k][j] = diff2[k][j] = delta;
@@ -1010,8 +1138,6 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
             /* now scanc-1 is next color which must be written explicitly */
             if (scanc - curc > 2) { /* good, we have a shaded range */
                if (scanc != maxcolor) {
-                  static FCODE msg[] = {"Tell Tim Wegner to check diffmag MISCOVL.C"};
-                  if(diffmag < 0) stopmsg(0,msg);
                   if (diffmag < 3) {  /* not a sharp slope change? */
                      force = 2;       /* force more between ranges, to stop  */
                      --scanc;         /* "drift" when load/store/load/store/ */
@@ -1032,27 +1158,6 @@ static void write_batch_parms(char *colorinf, int maxcolor, int ii, int jj)
          }
       }
 
-   if (rotate_lo != 1 || rotate_hi != 255)
-      put_parm( s_seqdd,s_cyclerange,rotate_lo,rotate_hi);
-
-   if(basehertz != 440)
-      put_parm(s_seqd,s_hertz,basehertz);
-
-   if(start_showorbit != 0)
-
-   if(soundflag == 1)
-      put_parm(s_seqs,s_sound,s_x);
-   else if(soundflag == 2)
-      put_parm(s_seqs,s_sound,s_y);
-   else if(soundflag == 3)
-      put_parm(s_seqs,s_sound,s_z);
-
-   if(orbit_delay > 0)
-      put_parm(s_seqd,s_orbitdelay,orbit_delay);
-
-   if(start_showorbit > 0)
-      put_parm(s_seqs,s_showorbit,s_yes);
-
    while (wbdata->len) /* flush the buffer */
       put_parm_line();
    /* restore previous boxx data from extraseg */
@@ -1099,8 +1204,9 @@ va_dcl
       put_parm_line();
 }
 
-#define NICELINELEN 68
-#define MAXLINELEN  72
+int maxlinelength=72;
+#define MAXLINELEN  maxlinelength
+#define NICELINELEN (MAXLINELEN-4)
 
 static void put_parm_line()
 {
@@ -1189,7 +1295,7 @@ int getprecbf(int rezflag)
    bfyydel2  = alloc_stack(bflength+2);
    floattobf(one,1.0);
    if(rezflag == MAXREZ)
-      rez = MAXPIXELS -1;
+      rez = OLDMAXPIXELS -1;
    else
       rez = xdots-1;
 
@@ -1246,7 +1352,7 @@ int getprecdbl(int rezflag)
    int digits;
    LDBL rez;
    if(rezflag == MAXREZ)
-      rez = MAXPIXELS -1;
+      rez = OLDMAXPIXELS -1;
    else
       rez = xdots-1;
 
@@ -1348,7 +1454,7 @@ void shell_to_dos()
 {
    int drv;
    char *comspec;
-   char curdir[80],*s;
+   char curdir[FILE_MAX_DIR],*s;
    if ((comspec = getenv("COMSPEC")) == NULL)
       printf("Cannot find COMMAND.COM.\n");
    else {
@@ -1365,16 +1471,7 @@ void shell_to_dos()
 
 size_t showstack(void)
 {
-#ifdef _MSC_VER
-#if (_MSC_VER == 700 || _MSC_VER == 800)
-   return(_stackavail());
-/* add other implementations here */
-#elif (_MSC_VER == 600)
    return(stackavail());
-#endif
-#else
-   return(-1);
-#endif
 }
 
 long fr_farfree(void)
@@ -1399,8 +1496,8 @@ void showfreemem(void)
    char adapter_name[8];        /* entry lenth from VIDEO.ASM */
    char *adapter_ptr;
 
-   printf("\n CPU type: %d  FPU type: %d  IIT FPU: %d  Video: %d",
-          cpu, fpu, iit, video_type);
+   printf("\n CPU type: %d  FPU type: %d  Video: %d",
+          cpu, fpu, video_type);
 
    adapter_ptr = &supervga_list;
 
@@ -1882,6 +1979,7 @@ if (xstep == 0 && ystep == 0) {         /* first time through? */
                 temp[4] = '7';
                 temp[5] = 'a';
                 }
+            temp[12] = 0; /* reserved */
             if (fwrite(temp,13,1,out) != 1)     /* write out the header */
                 errorflag = 1;
             }                           /* end of first-time-through */
@@ -1910,6 +2008,7 @@ if (xstep == 0 && ystep == 0) {         /* first time through? */
             }
 
         for (;;) {                      /* process each information block */
+        memset(temp,0,10);
         if (fread(temp,1,1,in) != 1)    /* read the block identifier */
             inputerrorflag = 3;
 
@@ -2221,7 +2320,7 @@ static char *expand_var(char *var, char *buf)
    }
    else if(far_strcmp(var,s_calctime) == 0)
    {
-      get_calculation_time(buf);
+      get_calculation_time(buf,calctime);
       out = buf;
    }
    else if(far_strcmp(var,s_version) == 0)  /* 4 chars */
