@@ -161,6 +161,7 @@ OLD_FN  dStkCeil;
 OLD_FN  dStkTrunc;
 OLD_FN  dStkRound;
 OLD_FN  StkJump, dStkJumpOnTrue, dStkJumpOnFalse;
+OLD_FN  dStkOne;
 
 typedef void (near NEW_FN)(void);  /* new 387-only ASM functions  */
 
@@ -220,6 +221,7 @@ NEW_FN  fStkATanh, fStkATan;
 NEW_FN  fStkCAbs;
 NEW_FN  fStkFloor, fStkCeil, fStkTrunc, fStkRound; /* rounding functions */
 NEW_FN  fStkJump, fStkJumpOnTrue, fStkJumpOnFalse, fStkJumpLabel; /* flow */
+NEW_FN  fStkOne;   /* to support new parser fn.  */
 
 /* check to see if a const is being loaded  */
 /* the really awful hack below gets the first char of the name  */
@@ -264,7 +266,7 @@ int pstopmsg(int x,char far *msg)
       fprintf(fp,"%Fs\n",msg);
       fflush(fp);
    }
-   return(x); /* just to quiet warnings */   
+   return(x); /* just to quiet warnings */
 }
 
 #define stopmsg pstopmsg
@@ -356,17 +358,18 @@ int pstopmsg(int x,char far *msg)
 #define FN_JUMP_ON_TRUE  52
 #define FN_JUMP_ON_FALSE 53
 #define FN_JUMP_LABEL    54
+#define FN_ONE           55
 
 
 /* number of "old" functions in the table.  */
 /* these are the ones that the parser outputs  */
 
-#define LAST_OLD_FN   FN_JUMP_LABEL
+#define LAST_OLD_FN   FN_ONE
 #define NUM_OLD_FNS   LAST_OLD_FN + 1
 
 /* total number of functions in the table.  */
 
-#define LAST_FN          FN_JUMP_LABEL
+#define LAST_FN          FN_ONE
 #define NUM_FNS          LAST_FN + 1
 
 static unsigned char
@@ -461,7 +464,8 @@ struct fn_entry {
    {FNAME("Jump",        StkJump,         fStkJump,       0, 0, 0)},/* 51  */
    {FNAME("JumpOnTrue",  dStkJumpOnTrue,  fStkJumpOnTrue, 2, 0, 0)},/* 52  */
    {FNAME("JumpOnFalse", dStkJumpOnFalse, fStkJumpOnFalse,2, 0, 0)},/* 53  */
-   {FNAME("JumpLabel",   StkJumpLabel,    fStkJumpLabel,  0, 0, 0)} /* 54  */
+   {FNAME("JumpLabel",   StkJumpLabel,    fStkJumpLabel,  0, 0, 0)},/* 54  */
+   {FNAME("One",     dStkOne,     fStkOne,    2, 0,  0) }           /* 55  */
 };
 
 #ifdef COMPILER
