@@ -311,7 +311,7 @@ start:
       out = i;
    if(out == -12345)
    {
-      char msg[80];
+      char msg[MSGLEN];
       sprintf(msg,s_cantunderstand,buffer);
       slideshowerr(msg);
       out = 0;
