@@ -7,6 +7,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/file.h>
@@ -15,6 +16,12 @@
 #include <ctype.h>
 #include "port.h"
 
+#define FILE_MAX_PATH  256       /* max length of path+filename  */
+#define FILE_MAX_DIR   256       /* max length of directory name */
+#define FILE_MAX_DRIVE  3       /* max length of drive letter   */
+#define FILE_MAX_FNAME  9       /* max length of filename       */
+#define FILE_MAX_EXT    5       /* max length of extension      */
+
 int iocount;
 
 /*
@@ -403,9 +410,9 @@ splitpath(char *template,char *drive,char *dir,char *fname,char *ext)
       {
 	 tmp++;  /* first character after slash */
 	 len = tmp - &template[offset];
-	 if(len >=0 && len < 80 && dir)
-	    strncpy(dir,&template[offset],len);
-	 if(len < 80 && dir)
+	 if(len >=0 && len < FILE_MAX_DIR && dir)
+	    strncpy(dir,&template[offset],min(len,FILE_MAX_DIR));
+	 if(len < FILE_MAX_DIR && dir)
 	    dir[len] = 0;
 	 offset += len;
       }
@@ -421,19 +428,28 @@ splitpath(char *template,char *drive,char *dir,char *fname,char *ext)
 	 tmp = 0; /* in this case the '.' must be a directory */
       if(tmp)
       {
-	 tmp++; /* first character past "." */
+	 /* tmp++; */ /* first character past "." */
 	 len = tmp - &template[offset];
 	 if((len > 0) && (offset+len < length) && fname)
 	 {
-	    strncpy(fname,&template[offset],len);
-	    fname[len] = 0;
+	    strncpy(fname,&template[offset],min(len,FILE_MAX_FNAME));
+            if(len < FILE_MAX_FNAME)
+               fname[len] = 0;
+            else
+               fname[FILE_MAX_FNAME-1] = 0;
 	 }
 	 offset += len;
 	 if((offset < length) && ext)
-	    strcpy(ext,&template[offset]);
+	 {
+	    strncpy(ext,&template[offset],FILE_MAX_EXT);
+            ext[FILE_MAX_EXT-1] = 0;
+	 }
       }
       else if((offset < length) && fname)
-	 strcpy(fname,&template[offset]);
+      {
+         strncpy(fname,&template[offset],FILE_MAX_FNAME);
+         fname[FILE_MAX_FNAME-1] = 0;
+      }
    }
    return(0);
 }
@@ -466,3 +482,53 @@ unsigned short _rotl(unsigned short num, short bits)
    ll = (((unsigned long)num << 16) + num) << (bits&15);
    return((unsigned short)(ll>>16)); 
 }
+
+/* sound.c file prototypes */
+int get_sound_params(void)
+{
+}
+void soundon(int i)
+{
+}
+void soundoff(void)
+{
+}
+
+void mute(void)
+{
+}
+
+int initfm(void)
+{
+  return(0);
+}
+
+/* tenths of millisecond timewr routine */
+static struct timeval tv_start;
+
+void restart_uclock(void)
+{
+   gettimeofday(&tv_start, NULL);
+}
+
+typedef unsigned long uclock_t;
+uclock_t usec_clock(void)
+{
+   uclock_t result;
+
+   struct timeval tv, elapsed;
+   gettimeofday(&tv, NULL);
+
+   elapsed.tv_usec  = tv.tv_usec -  tv_start.tv_sec;
+   elapsed.tv_sec   = tv.tv_sec -   tv_start.tv_sec;
+
+   if(elapsed.tv_usec < 0)
+   {
+      /* "borrow */
+      elapsed.tv_usec += 1000000;
+      elapsed.tv_sec--;
+   }
+   result  = (unsigned long)(elapsed.tv_sec*10000 +  elapsed.tv_usec/100);
+   return(result); 
+}
+
