@@ -42,9 +42,7 @@ typedef unsigned int UINT4;
 /* inline is a reserved word */
 #define inline inline1
 
-#ifndef LINUX
 typedef void (*SignalHandler)(int);
-#endif
 
 /* Some stdio.h's don't have this */
 #ifndef SEEK_SET
@@ -82,6 +80,10 @@ char *strupr(char *s);
 #endif
 #endif
 
+/* For Unix, all memory is FARMEM */
+#define EXPANDED FARMEM
+#define EXTENDED FARMEM
+
 /*
  * These defines are so movedata, etc. will work properly, without worrying
  * about the silly segment stuff.
