@@ -241,7 +241,7 @@ UnixInit()
     initdacbox();
 
     if (!simple_input) {
-	  signal(SIGINT,(SignalHandler)goodbye);
+	  signal(SIGINT,(__sighandler_t)goodbye);
     }
     signal(SIGFPE, fpe_handler);
     /*
@@ -2068,12 +2068,12 @@ xgetfont()
 void
 shell_to_dos()
 {
-    SignalHandler sigint;
+    __sighandler_t sigint;
     char *shell;
     char *argv[2];
     int pid, donepid;
 
-    sigint = (SignalHandler)signal(SIGINT, SIG_IGN);
+    sigint = (__sighandler_t)signal(SIGINT, SIG_IGN);
     shell = getenv("SHELL");
     if (shell==NULL) {
 	shell = SHELL;
@@ -2121,7 +2121,7 @@ shell_to_dos()
 	fcntl(0,F_SETFL,FNDELAY);
     }
 
-    signal(SIGINT, (SignalHandler)sigint);
+    signal(SIGINT, (__sighandler_t)sigint);
     putchar('\n');
 }
 
@@ -2146,7 +2146,7 @@ schedulealarm(soon)
 int soon;
 {
     if (!fastmode) return;
-    signal(SIGALRM, (SignalHandler)setredrawscreen);
+    signal(SIGALRM, (__sighandler_t)setredrawscreen);
     if (soon) {
 	alarm(1);
     } else {
