@@ -1,23 +1,16 @@
 #include <curses.h>
 #include <string.h>
-#include "fractint.h"
+#include "port.h"
 #include "prototyp.h"
-
 /*
  * This file contains Unix versions of the routines in video.asm
  * Copyright 1992 Ken Shirriff
  */
 
-extern int colors,rotate_lo,rotate_hi;
-extern int boxx[],boxy[],boxcount;
-
 extern unsigned char *xgetfont();
 
 WINDOW *curwin;
 
-extern int boxcount;
-extern int dotmode;
-extern int sxdots,sydots,sxoffs,syoffs;
 int daclearn = 0;
 int dacnorm = 0;
 int daccount = 0;
@@ -226,12 +219,12 @@ int xdot,ydot;
 }
 
 /*
-; ************** Function putcolor(xdot, ydot, color) *******************
+; ************** Function putcolor_a(xdot, ydot, color) *******************
 
 ;       write the color on the screen at the (xdot,ydot) point
 */
 void
-putcolor(xdot,ydot,color)
+putcolor_a(xdot,ydot,color)
 int xdot,ydot,color;
 {
     dotwrite(xdot+sxoffs,ydot+syoffs,color&andcolor);
@@ -574,7 +567,7 @@ void
 find_special_colors()
 {
     int maxb = 0;
-    int minb = 99999;
+    int minb = 9999;
     int med = 0;
     int maxgun, mingun;
     int brt;
@@ -593,7 +586,7 @@ find_special_colors()
     if (!gotrealdac) return;
 
     for (i=0;i<colors;i++) {
-	brt = ((unsigned char *)dacbox)[i*3+0]+((unsigned char *)dacbox)[i*3+1]+((unsigned char *)dacbox)[i*3+2];
+	brt = (int)dacbox[i][0] + (int)dacbox[i][1] + (int)dacbox[i][2];
 	if (brt>maxb) {
 	    maxb = brt;
 	    color_bright = i;
@@ -603,25 +596,17 @@ find_special_colors()
 	    color_dark = i;
 	}
 	if (brt<150 && brt>80) {
-	    maxgun = mingun = (unsigned char)dacbox[i*3+0];
-	    if ((unsigned char)dacbox[i*3+1]>(unsigned char)dacbox[i*3+0]) {
-		maxgun = (unsigned char)dacbox[i*3+1];
+	    maxgun = mingun = (int)dacbox[i][0];
+	    if ((int)dacbox[i][1]>(int)dacbox[i][0]) {
+		maxgun = (int)dacbox[i][1];
 	    } else {
-		mingun = (unsigned char)dacbox[i*3+1];
+		mingun = (int)dacbox[i][1];
 	    }
-#ifdef __SVR4
-	    if((int)((unsigned char)dacbox[i*3+2])>maxgun) {
-#else
-	    if((unsigned char)dacbox[i*3+2]>maxgun) {
-#endif
-		maxgun = (unsigned char)dacbox[i*3+2];
+	    if((int)dacbox[i][2] > maxgun) {
+		maxgun = (int)dacbox[i][2];
 	    }
-#ifdef __SVR4
-	    if((int)((unsigned char)dacbox[i*3+2])<mingun) {
-#else
-	    if((unsigned char)dacbox[i*3+2]<mingun) {
-#endif
-		mingun = (unsigned char)dacbox[i*3+2];
+	    if((int)dacbox[i][2] < mingun) {
+		mingun = (int)dacbox[i][2];
 	    }
 	    if (brt-(maxgun-mingun)/2>med){
 		color_medium = i;
