@@ -35,6 +35,55 @@ void calc_corner(bf_t target,bf_t p1,double p2,bf_t p3,double p4,bf_t p5)
    restore_stack(saved);
 }
 
+int boxcolor;
+
+#ifndef XFRACT
+void dispbox(void)
+{
+   int i;
+   xorTARGA = 1;
+   for(i=0;i<boxcount;i++)
+   {
+      if(istruecolor)
+      {
+         int rgb[3];
+         gettruecolor(boxx[i]-sxoffs,
+                      boxy[i]-syoffs,&rgb[0],&rgb[1],&rgb[2]);
+         puttruecolor(boxx[i]-sxoffs,
+                      boxy[i]-syoffs,rgb[0]^255,rgb[1]^255,rgb[2]^255);
+      }
+      else
+      {
+         unsigned char *values = (unsigned char *)boxvalues;
+         int boxc = (colors-1)&boxcolor;
+         values[i] = (unsigned char)
+         getcolor(boxx[i]-sxoffs,boxy[i]-syoffs);
+         putcolor(boxx[i]-sxoffs,boxy[i]-syoffs,boxc);
+      }
+   }
+   xorTARGA = 0;
+}
+
+void clearbox(void)
+{
+   int i;
+   xorTARGA = 1;
+   if(istruecolor)
+   {
+      dispbox();
+   }
+   else
+   {
+      unsigned char *values = (unsigned char *)boxvalues;
+      for(i=0;i<boxcount;i++)
+      {
+         putcolor(boxx[i]-sxoffs,boxy[i]-syoffs,values[i]);
+      }
+   }
+   xorTARGA = 0;
+}
+#endif
+
 void drawbox(int drawit)
 {
     struct coords tl,bl,tr,br; /* dot addr of topleft, botleft, etc */
@@ -44,7 +93,7 @@ void drawbox(int drawit)
     int saved;
     if (zwidth==0) { /* no box to draw */
         if (boxcount!=0) { /* remove the old box from display */
-            clearbox();   /* asm routine */
+            clearbox(); 
             boxcount = 0; }
         reset_zoom_corners();
         return; }
@@ -133,7 +182,7 @@ void drawbox(int drawit)
     tr.y   = (int)(ftemp2*(dysize+PIXELROUND));
 
     if (boxcount!=0) { /* remove the old box from display */
-        clearbox();   /* asm routine */
+        clearbox(); 
         boxcount = 0; }
 
     if (drawit) { /* caller wants box drawn as well as co-ords calc'd */
@@ -154,7 +203,7 @@ void drawbox(int drawit)
         boxy[3] = bl.y + syoffs;
         boxcount = 1;
 #endif
-        dispbox();                            /* asm routine to paint it */
+        dispbox();
         }
     }
 
@@ -219,7 +268,8 @@ void _fastcall addbox(struct coords point)
 {
     point.x += sxoffs;
     point.y += syoffs;
-    if (point.x >= 0 && point.x < sxdots && point.y >= 0 && point.y < sydots) {
+    if (point.x >= 0 && point.x < sxdots && 
+        point.y >= 0 && point.y < sydots) {
         boxx[boxcount] = point.x;
         boxy[boxcount] = point.y;
         ++boxcount;
@@ -482,7 +532,7 @@ void aspectratio_crop(float oldaspect,float newaspect)
 
 static int check_pan(void) /* return 0 if can't, alignment requirement if can */
 {   int i,j;
-    if (calc_status != 2 && calc_status != 4)
+    if ((calc_status != 2 && calc_status != 4) || evolving)
         return(0); /* not resumable, not complete */
     if ( curfractalspecific->calctype != StandardFractal
       && curfractalspecific->calctype != calcmand
@@ -501,6 +551,8 @@ static int check_pan(void) /* return 0 if can't, alignment requirement if can */
         return(1); /* btm, align on any pixel */
     if (stdcalcmode == 't')
         return(0); /* tesselate, can't do it */
+    if (stdcalcmode == 'd')
+	return(0); /* diffusion scan: can't do it either */
     if (stdcalcmode != 'g' || (curfractalspecific->flags&NOGUESS)) {
         if (stdcalcmode == '2' || stdcalcmode == '3') /* align on even pixel for 2pass */
            return(2);
@@ -542,7 +594,7 @@ int init_pan_or_recalc(int do_zoomout) /* decide to recalc, or to chg worklist &
     if (zwidth == 0.0)
         return(0); /* no zoombox, leave calc_status as is */
     /* got a zoombox */
-    if ((alignmask=check_pan()-1) < 0) {
+    if ((alignmask=check_pan()-1) < 0 || evolving) {
         calc_status = 0; /* can't pan, trigger recalc */
         return(0); }
     if (zbx == 0.0 && zby == 0.0) {
@@ -569,20 +621,21 @@ int init_pan_or_recalc(int do_zoomout) /* decide to recalc, or to chg worklist &
         worklist[i].yybegin -= row;
         worklist[i].xxstart -= col;
         worklist[i].xxstop  -= col;
+        worklist[i].xxbegin -= col;
         }
     /* add worklist entries for the new edges */
     listfull = i = 0;
     j = ydots-1;
     if (row < 0) {
-        listfull |= add_worklist(0,xdots-1,0,0-row-1,0,0,0);
+        listfull |= add_worklist(0,xdots-1,0,0,0-row-1,0,0,0);
         i = 0 - row; }
     if (row > 0) {
-        listfull |= add_worklist(0,xdots-1,ydots-row,ydots-1,ydots-row,0,0);
+        listfull |= add_worklist(0,xdots-1,0,ydots-row,ydots-1,ydots-row,0,0);
         j = ydots - row - 1; }
     if (col < 0)
-        listfull |= add_worklist(0,0-col-1,i,j,i,0,0);
+        listfull |= add_worklist(0,0-col-1,0,i,j,i,0,0);
     if (col > 0)
-        listfull |= add_worklist(xdots-col,xdots-1,i,j,i,0,0);
+        listfull |= add_worklist(xdots-col,xdots-1,xdots-col,i,j,i,0,0);
     if (listfull != 0) {
     static FCODE msg[] = {"\
 Tables full, can't pan current image.\n\
@@ -618,6 +671,7 @@ static void _fastcall restart_window(int wknum)
         put_line(yfrom++,xfrom,xto,(BYTE *)dstack);
     worklist[wknum].sym = worklist[wknum].pass = 0;
     worklist[wknum].yybegin = worklist[wknum].yystart;
+    worklist[wknum].xxbegin = worklist[wknum].xxstart;
 }
 
 static void fix_worklist(void) /* fix out of bounds and symmetry related stuff */
@@ -688,6 +742,8 @@ static void fix_worklist(void) /* fix out of bounds and symmetry related stuff *
            wk->xxstop = j; }
         if (wk->yybegin < wk->yystart) wk->yybegin = wk->yystart;
         if (wk->yybegin > wk->yystop)  wk->yybegin = wk->yystop;
+        if (wk->xxbegin < wk->xxstart) wk->xxbegin = wk->xxstart;
+        if (wk->xxbegin > wk->xxstop)  wk->xxbegin = wk->xxstop;
         }
     tidy_worklist(); /* combine where possible, re-sort */
 }
